@@ -1,14 +1,18 @@
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
-#include <gtk/gtk.h>
+/*
+ *  This program is free software; you can redistribute it and/or
+ *  modify it under the terms of the GNU General Public License as
+ *  published by the Free Software Foundation; either version 3 of
+ *  the License, or (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details:
+ *
+ *  http://www.gnu.org/copyleft/gpl.txt
+ */
 
 #include "callbacks.h"
-#include "interface.h"
-#include "support.h"
-#include "common.h"
-#include "cat.h"
 #include "shared.h"
 
 
@@ -17,7 +21,7 @@ static GtkWidget
   *quit_dialog  = NULL,
   *save_dialog  = NULL,
   *error_dialog = NULL,
-  *bad_entry_dialog = NULL;
+  *entry_dialog = NULL;
 
 static int newc_ptr = 0;
 
@@ -30,62 +34,72 @@ static int newc_ptr = 0;
   void
 Error_Dialog( char *mesg, gboolean hide )
 {
+  GtkBuilder *builder = NULL;
   if( error_dialog != NULL ) return;
-  error_dialog = create_error_dialog();
-  GtkWidget *lbl = lookup_widget( error_dialog, "error_message" );
+  error_dialog = create_error_dialog( &builder );
+  GtkWidget *lbl = Builder_Get_Object( builder, "error_message" );
   gtk_label_set_text( GTK_LABEL(lbl), mesg );
   if( hide )
-	gtk_widget_hide( lookup_widget(error_dialog, "error_ok_button") );
+	gtk_widget_hide( Builder_Get_Object(builder, "error_ok_button") );
   gtk_widget_show( error_dialog );
+  g_object_unref( builder );
 }
 
 
   void
 Quit_Dialog( char *mesg )
 {
+  GtkBuilder *builder = NULL;
   if( quit_dialog != NULL ) return;
-  quit_dialog = create_quit_dialog();
-  GtkWidget *lbl = lookup_widget( quit_dialog, "quit_message" );
+  quit_dialog = create_quit_dialog( &builder );
+  GtkWidget *lbl = Builder_Get_Object( builder, "quit_message" );
   gtk_label_set_text( GTK_LABEL(lbl), mesg );
   gtk_widget_show( quit_dialog );
+  g_object_unref( builder );
 }
 
 
   void
 Save_Dialog( char *mesg )
 {
+  GtkBuilder *builder = NULL;
   if( save_dialog != NULL ) return;
-  save_dialog = create_save_dialog();
-  GtkWidget *lbl = lookup_widget( save_dialog, "save_message" );
+  save_dialog = create_save_dialog( &builder );
+  GtkWidget *lbl = Builder_Get_Object( builder, "save_message" );
   gtk_label_set_text( GTK_LABEL(lbl), mesg );
   gtk_widget_show( save_dialog );
+  g_object_unref( builder );
 }
 
 
   void
 Bad_Entry_Dialog( char *mesg )
 {
-  if( bad_entry_dialog != NULL ) return;
-  bad_entry_dialog = create_bad_entry_dialog();
-  GtkWidget *lbl = lookup_widget( bad_entry_dialog, "bad_entry" );
+  GtkBuilder *builder = NULL;
+  if( entry_dialog != NULL ) return;
+  entry_dialog = create_entry_dialog( &builder );
+  GtkWidget *lbl = Builder_Get_Object( builder, "entry_error" );
   gtk_label_set_text( GTK_LABEL(lbl), mesg );
-  gtk_widget_show( bad_entry_dialog );
+  gtk_widget_show( entry_dialog );
+  g_object_unref( builder );
 }
 
 
-void
-on_main_window_destroy                 (GtkObject       *object,
-                                        gpointer         user_data)
+  void
+on_main_window_destroy(
+	GObject  *object,
+	gpointer  user_data)
 {
   Cleanup();
   gtk_main_quit();
 }
 
 
-gboolean
-on_main_window_delete_event            (GtkWidget       *widget,
-                                        GdkEvent        *event,
-                                        gpointer         user_data)
+  gboolean
+on_main_window_delete_event(
+	GtkWidget *widget,
+	GdkEvent  *event,
+	gpointer   user_data)
 {
   if( isFlagSet(TRANSMIT_MODE) ||
 	  isFlagSet(RECEIVE_MODE) )
@@ -93,17 +107,17 @@ on_main_window_delete_event            (GtkWidget       *widget,
 	Quit_Dialog(
 		_("xfhell is running\n"\
 		  "Do you want to quit?") );
-	return TRUE;
+	return( TRUE );
   }
-  else return FALSE;
+  else return( FALSE );
 }
 
 
   gboolean
 on_tx_textview_button_press_event(
-	GtkWidget       *widget,
-	GdkEventButton  *event,
-	gpointer         user_data)
+	GtkWidget      *widget,
+	GdkEventButton *event,
+	gpointer        user_data)
 {
   switch( event->button )
   {
@@ -129,26 +143,28 @@ on_tx_textview_button_press_event(
 	  break;
 
 	case 3: /* Popup menu */
-	  if( isFlagSet(PLAYBACK_SETUP) && isFlagSet(TRANSMIT_KEYBD) )
-		gtk_widget_set_sensitive( lookup_widget(gbl_popup_menu,
-			  "identify_in_cw"), TRUE );
+	  if( isFlagSet(PLAYBACK_SETUP) &&
+		  isFlagSet(TRANSMIT_KEYBD) )
+		gtk_widget_set_sensitive( Builder_Get_Object(
+			  popup_menu_builder, "identify_in_cw"), TRUE );
 	  else
-		gtk_widget_set_sensitive( lookup_widget(gbl_popup_menu,
-			  "identify_in_cw"), FALSE );
+		gtk_widget_set_sensitive( Builder_Get_Object(
+			  popup_menu_builder, "identify_in_cw"), FALSE );
 
-	  gtk_menu_popup( GTK_MENU(gbl_popup_menu),
-		  NULL, NULL, NULL, NULL, 3, 0);
+	  gtk_menu_popup_at_pointer( GTK_MENU(popup_menu), NULL );
 	  break;
+
   } /* switch( event->button ) */
 
-  return FALSE;
+  return( TRUE );
 }
 
 
-gboolean
-on_tx_textview_key_press_event         (GtkWidget       *widget,
-                                        GdkEventKey     *event,
-                                        gpointer         user_data)
+  gboolean
+on_tx_textview_key_press_event(
+	GtkWidget   *widget,
+	GdkEventKey *event,
+	gpointer     user_data)
 {
   /* Text buffer marker */
   static GtkTextIter txtbuf_iter;
@@ -163,7 +179,7 @@ on_tx_textview_key_press_event         (GtkWidget       *widget,
 	return( FALSE );
 
   /* Deal with backspace */
-  if( (event->keyval == GDK_BackSpace) && char_cnt )
+  if( (event->keyval == GDK_KEY_BackSpace) && char_cnt )
   {
 	/* Reduce char cnt for word wrap */
 	char_cnt--;
@@ -187,12 +203,12 @@ on_tx_textview_key_press_event         (GtkWidget       *widget,
 	keybd_buff[newc_ptr] = '\0';
 
 	Print_Character(
-		BS, gbl_tx_text_buffer,
-		gbl_tx_scrolledwindow,
+		BS, tx_text_buffer,
+		tx_scrolledwindow,
 		&txtbuf_iter );
 
 	return( TRUE );
-  } /* if( (event->keyval == GDK_BackSpace) && char_cnt ) */
+  } /* if( (event->keyval == GDK_KEY_BackSpace) && char_cnt ) */
 
   /* Enter new caracter to key buffer */
   keybd_buff[newc_ptr] = event->keyval;
@@ -206,16 +222,16 @@ on_tx_textview_key_press_event         (GtkWidget       *widget,
 
   /* Set up TRANSMIT text view */
   gtk_text_buffer_get_iter_at_offset(
-	  gbl_tx_text_buffer, &txtbuf_iter,
+	  tx_text_buffer, &txtbuf_iter,
 	  gtk_text_buffer_get_char_count
-	  (gbl_tx_text_buffer) );
+	  (tx_text_buffer) );
 
   /* Print ascii characters to screen */
   if( keybd_buff[newc_ptr] <= 0xff )
 	Print_Character(
 		(int)keybd_buff[newc_ptr],
-		gbl_tx_text_buffer,
-		gbl_tx_scrolledwindow,
+		tx_text_buffer,
+		tx_scrolledwindow,
 		&txtbuf_iter );
 
   /* Set keybd busy flag since a space */
@@ -236,16 +252,16 @@ on_tx_textview_key_press_event         (GtkWidget       *widget,
 	{
 	  char_cnt = 0; /* Reset word wrap */
 	  Print_Character(
-		  GDK_Return,
-		  gbl_tx_text_buffer,
-		  gbl_tx_scrolledwindow,
+		  GDK_KEY_Return,
+		  tx_text_buffer,
+		  tx_scrolledwindow,
 		  &txtbuf_iter );
 	}
 
   } /* if( keybd_buff[newc_ptr] == ' ' ) */
 
   /* Send a space in place of Return */
-  if( keybd_buff[newc_ptr] == GDK_Return )
+  if( keybd_buff[newc_ptr] == GDK_KEY_Return )
   {
 	keybd_buff[newc_ptr] = ' ';
 	if( isFlagClear(RECEIVE_MODE) )
@@ -273,13 +289,14 @@ on_tx_textview_key_press_event         (GtkWidget       *widget,
   if( --newc_ptr < 0 )
 	newc_ptr += KEY_BUFF_SIZE;
 
-  return FALSE;
+  return( FALSE );
 }
 
 
-void
-on_txvol_spinbutton_value_changed      (GtkSpinButton   *spinbutton,
-                                        gpointer         user_data)
+  void
+on_txvol_spinbutton_value_changed(
+	GtkSpinButton *spinbutton,
+	gpointer       user_data)
 {
   char mesg[MESG_SIZE];
   int error, spin_val;
@@ -287,11 +304,11 @@ on_txvol_spinbutton_value_changed      (GtkSpinButton   *spinbutton,
   spin_val = gtk_spin_button_get_value_as_int(spinbutton);
 
   /* If tcvr is K2 or K3, set power output setting */
-  if( (rc_data.tcvr_type == K2) || (rc_data.tcvr_type == K3) )
+  if( (rc_data.tcvr_type == K2) ||
+	  (rc_data.tcvr_type == K3) )
   {
 	rc_data.tx_output = spin_val;
-	if( isFlagSet(CAT_SETUP) )
-	  Set_Tx_Power( rc_data.tx_output, 0 );
+	if( isFlagSet(CAT_SETUP) ) Set_Tx_Power( rc_data.tx_output, 0 );
 	rc_data.pbk_lev = K3_PBK_LEVEL;
   }
   else rc_data.pbk_lev = spin_val;
@@ -301,29 +318,41 @@ on_txvol_spinbutton_value_changed      (GtkSpinButton   *spinbutton,
 }
 
 
-gboolean
-on_drawingarea_expose_event            (GtkWidget       *widget,
-                                        GdkEventExpose  *event,
-                                        gpointer         user_data)
+  void
+on_rx_drawingarea_configure_event(
+	GtkWidget *widget,
+	GdkEventConfigure *event,
+	gpointer user_data )
 {
-  if( receive_pixbuf.pixbuf == NULL )
-	return FALSE;
-
-  gdk_draw_pixbuf(
-	  widget->window, NULL, receive_pixbuf.pixbuf,
-	  event->area.x, event->area.y,
-	  event->area.x, event->area.y,
-	  event->area.width, event->area.height,
-	  GDK_RGB_DITHER_NONE, 0, 0 );
-
-  return FALSE;
+  rc_data.rxarea_width  = event->width;
+  rc_data.rxarea_height = event->height;
+  if( !Create_Rx_Pixbuf() )
+	Error_Dialog( _("Failed to create pixbuf for Rx window"), QUIT );
+  SetFlag( CLEAR_RX_WINDOW );
 }
 
 
-gboolean
-on_drawingarea_button_press_event      (GtkWidget       *widget,
-                                        GdkEventButton  *event,
-                                        gpointer         user_data)
+  gboolean
+on_rx_drawingarea_draw(
+	GtkWidget *widget,
+	cairo_t   *cr,
+	gpointer   user_data)
+{
+  if( receive_pixbuf.pixbuf != NULL )
+  {
+	gdk_cairo_set_source_pixbuf( cr, receive_pixbuf.pixbuf, 0.0, 0.0 );
+	cairo_paint( cr );
+	return( TRUE );
+  }
+  return( FALSE );
+}
+
+
+  gboolean
+on_rx_drawingarea_button_press_event(
+	GtkWidget      *widget,
+	GdkEventButton *event,
+	gpointer        user_data)
 {
   switch( event->button )
   {
@@ -354,35 +383,24 @@ on_drawingarea_button_press_event      (GtkWidget       *widget,
 	case 3: /* Popup menu */
 	  if( isFlagSet(PLAYBACK_SETUP) &&
 		  isFlagSet(TRANSMIT_KEYBD) )
-		gtk_widget_set_sensitive( lookup_widget
-			(gbl_popup_menu, "identify_in_cw"), TRUE );
+		gtk_widget_set_sensitive( Builder_Get_Object(
+			  popup_menu_builder, "identify_in_cw"), TRUE );
 	  else
-		gtk_widget_set_sensitive( lookup_widget
-			(gbl_popup_menu, "identify_in_cw"), FALSE );
+		gtk_widget_set_sensitive( Builder_Get_Object(
+			  popup_menu_builder, "identify_in_cw"), FALSE );
 
-	  gtk_menu_popup( GTK_MENU(gbl_popup_menu),
-		  NULL, NULL, NULL, NULL, 3, 0);
+	  gtk_menu_popup_at_pointer( GTK_MENU(popup_menu), NULL );
 
   } /* switch( event->button ) */
 
-  return FALSE;
+  return( TRUE );
 }
 
 
-void
-on_drawingarea_size_allocate           (GtkWidget       *widget,
-                                        GdkRectangle    *allocation,
-                                        gpointer         user_data)
-{
-  if( !Create_Pixbuf(widget) )
-	Error_Dialog( _("Failed to create pixbuf for Rx window"), QUIT );
-  SetFlag( CLEAR_RX_WINDOW );
-}
-
-
-void
-on_xmit_clicked                        (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_xmit_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   if( isFlagSet(TRANSMIT_MACRO) )
 	ClearFlag( TRANSMIT_MACRO  );
@@ -404,9 +422,10 @@ on_xmit_clicked                        (GtkButton       *button,
 }
 
 
-void
-on_rcve_clicked                        (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_rcve_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   if( isFlagSet(RECEIVE_MODE) )
   {
@@ -433,9 +452,10 @@ on_rcve_clicked                        (GtkButton       *button,
 }
 
 
-void
-on_deskew_toggled                      (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
+  void
+on_deskew_toggled(
+	GtkToggleButton *togglebutton,
+	gpointer         user_data)
 {
   if( gtk_toggle_button_get_active(togglebutton) )
   {
@@ -453,24 +473,24 @@ on_deskew_toggled                      (GtkToggleButton *togglebutton,
 }
 
 
-void
-on_height_checkbutton_toggled          (GtkToggleButton *togglebutton,
-                                        gpointer         user_data)
+  void
+on_height_checkbutton_toggled(
+	GtkToggleButton *togglebutton,
+	gpointer         user_data)
 {
-  if( gbl_drawingarea == NULL ) return;
+  if( drawingarea == NULL ) return;
   if( gtk_toggle_button_get_active(togglebutton) )
-	gtk_widget_set_size_request( gbl_drawingarea,
-		gbl_drawingarea->allocation.width, 172 );
+	gtk_widget_set_size_request( drawingarea, rc_data.rxarea_width, 172 );
   else
-	gtk_widget_set_size_request( gbl_drawingarea,
-		gbl_drawingarea->allocation.width, 340 );
+	gtk_widget_set_size_request( drawingarea, rc_data.rxarea_width, 340 );
   SetFlag( INITIALIZE_CAIRO );
 }
 
 
-void
-on_rxvol_spinbutton_value_changed      (GtkSpinButton   *spinbutton,
-                                        gpointer         user_data)
+  void
+on_rxvol_spinbutton_value_changed(
+	GtkSpinButton *spinbutton,
+	gpointer       user_data)
 {
   char mesg[MESG_SIZE];
   int error;
@@ -484,107 +504,87 @@ on_rxvol_spinbutton_value_changed      (GtkSpinButton   *spinbutton,
 }
 
 
-void
-on_contrast_value_changed              (GtkRange        *range,
-                                        gpointer         user_data)
+  void
+on_contrast_value_changed(
+	GtkRange *range,
+	gpointer  user_data)
 {
-  GtkAdjustment *adjustment;
+  GtkAdjustment *adjust;
 
   /* Read and enter contrast value */
-  adjustment = gtk_range_get_adjustment( range );
-  contrast = ELEMENT_MAX+1 -
-	(int)(adjustment->value * ((double)ELEMENT_MAX/10.0));
-}
-
-
-gboolean
-on_scope_expose_event                  (GtkWidget       *widget,
-                                        GdkEventExpose  *event,
-                                        gpointer         user_data)
-{
-  if( wfall_pixbuf.pixbuf == NULL )
-	return FALSE;
-
-  if( isFlagClear(ENABLE_SCOPE) )
-  {
-	/* Draw waterfall */
-	gdk_draw_pixbuf(
-		widget->window, NULL, wfall_pixbuf.pixbuf,
-		event->area.x, event->area.y,
-		event->area.x, event->area.y,
-		event->area.width, event->area.height,
-		GDK_RGB_DITHER_NONE, 0, 0 );
-
-	gtk_label_set_text( GTK_LABEL(gbl_scope_label), "Waterfall" );
-  }
-
-  return FALSE;
+  adjust = gtk_range_get_adjustment( range );
+  contrast = ELEMENT_MAX + 1 -
+	(int)( gtk_adjustment_get_value(adjust) * ((double)ELEMENT_MAX/10.0) );
 }
 
 
   gboolean
-on_scope_button_press_event            (GtkWidget       *widget,
-										GdkEventButton  *event,
-                                        gpointer         user_data)
+on_scope_draw(
+	GtkWidget *widget,
+	cairo_t   *cr,
+	gpointer   user_data)
 {
-  switch( event->button )
-  {
-	case 1:
-	  if( isFlagSet(CAT_SETUP) &&
-		  isFlagSet(RECEIVE_MODE) &&
-		  isFlagClear(ENABLE_SCOPE) )
-		Tune_Tcvr( event->x );
-
-  } /* switch( event->button ) */
-
-  return FALSE;
-}
-
-
-void
-on_scope_size_allocate                 (GtkWidget       *widget,
-                                        GdkRectangle    *allocation,
-                                        gpointer         user_data)
-{
-  int idx;
-
-  /* Destroy existing pixbuff */
   if( wfall_pixbuf.pixbuf != NULL )
   {
-	g_object_unref( G_OBJECT(wfall_pixbuf.pixbuf) );
-	wfall_pixbuf.pixbuf = NULL;
+	if( isFlagClear(ENABLE_SCOPE) )
+	{
+	  gdk_cairo_set_source_pixbuf( cr, wfall_pixbuf.pixbuf, 0.0, 0.0 );
+	  cairo_paint( cr );
+	}
+	else Display_Signal( cr );
+	return( TRUE );
   }
-
-  /* Create waterfall pixbuf */
-  wfall_pixbuf.pixbuf = gdk_pixbuf_new(
-	  GDK_COLORSPACE_RGB, FALSE, 8,
-	  allocation->width, allocation->height );
-  if( wfall_pixbuf.pixbuf == NULL )
-	Error_Dialog( _("Failed to create pixbuf for waterfall"), QUIT );
-
-  wfall_pixbuf.pixels     = gdk_pixbuf_get_pixels( wfall_pixbuf.pixbuf );
-  wfall_pixbuf.width      = gdk_pixbuf_get_width ( wfall_pixbuf.pixbuf );
-  wfall_pixbuf.height     = gdk_pixbuf_get_height( wfall_pixbuf.pixbuf );
-  wfall_pixbuf.rowstride  = gdk_pixbuf_get_rowstride( wfall_pixbuf.pixbuf );
-  wfall_pixbuf.n_channels = gdk_pixbuf_get_n_channels( wfall_pixbuf.pixbuf );
-  Clear_Pixbuf( &wfall_pixbuf, 0 );
-  SetFlag( INITIALIZE_CAIRO );
-
-  /* Allocate average bin value buffer */
-  if( !mem_realloc((void **)&gbl_bin_ave,
-		(size_t)wfall_pixbuf.width * sizeof(int)) ) 
-	return;
-  for( idx = 0; idx < wfall_pixbuf.width; idx++ )
-	gbl_bin_ave[idx] = 0;
-
-  /* Initialize dft */
-  Idft_Init(DFT_INPUT_SIZE, wfall_pixbuf.width);
+  return( FALSE );
 }
 
 
-void
-on_rst_out_changed                     (GtkEditable     *editable,
-                                        gpointer         user_data)
+  gboolean
+on_scope_button_press_event(
+	GtkWidget      *widget,
+	GdkEventButton *event,
+	gpointer        user_data)
+{
+  if( event->button == 1 )
+  {
+	if( isFlagSet(CAT_SETUP) &&
+		isFlagSet(RECEIVE_MODE) &&
+		isFlagClear(ENABLE_SCOPE) )
+	  Tune_Tcvr( event->x );
+  }
+  else
+  {
+	if( isFlagSet( ENABLE_SCOPE ) )
+	{
+	  gtk_label_set_text( GTK_LABEL(scope_label), "Waterfall" );
+	  ClearFlag( ENABLE_SCOPE );
+	}
+	else
+	{
+	  gtk_label_set_text( GTK_LABEL(scope_label), _("Signal Level") );
+	  SetFlag( ENABLE_SCOPE );
+	}
+	gtk_widget_queue_draw( scope );
+  }
+
+  return( TRUE );
+}
+
+
+  void
+on_scope_configure_event(
+	GtkWidget    *widget,
+	GdkEventConfigure *event,
+	gpointer      user_data)
+{
+  if( !Create_Scope_Pixbuf(event->width, event->height) )
+	Error_Dialog( _("Failed to create pixbuf for waterfall"), QUIT );
+}
+
+
+  void
+on_rst_out_changed(
+	GtkEditable *editable,
+	gpointer    user_data)
 {
   Process_RST( editable );
   Strlcpy( qso_record.dx_rst,
@@ -593,9 +593,10 @@ on_rst_out_changed                     (GtkEditable     *editable,
 }
 
 
-void
-on_rst_in_changed                      (GtkEditable     *editable,
-                                        gpointer         user_data)
+  void
+on_rst_in_changed(
+	GtkEditable *editable,
+	gpointer     user_data)
 {
   Process_RST( editable );
   Strlcpy( qso_record.my_rst,
@@ -604,17 +605,17 @@ on_rst_in_changed                      (GtkEditable     *editable,
 }
 
 
-void
-on_op_name_changed                     (GtkEditable     *editable,
-                                        gpointer         user_data)
+  void
+on_op_name_changed(
+	GtkEditable *editable,
+	gpointer     user_data)
 {
   char buff[13];
   int len;
 
   /* Read entry, ignore blank field */
   Strlcpy( buff,
-	  gtk_entry_get_text(GTK_ENTRY(editable)),
-	  sizeof(buff) );
+	  gtk_entry_get_text(GTK_ENTRY(editable)), sizeof(buff) );
   len = (int)strlen(buff);
   if( len == 0 ) return;
 
@@ -634,9 +635,10 @@ on_op_name_changed                     (GtkEditable     *editable,
 }
 
 
-void
-on_qth_name_changed                    (GtkEditable     *editable,
-                                        gpointer         user_data)
+  void
+on_qth_name_changed(
+	GtkEditable *editable,
+	gpointer     user_data)
 {
   char buff[13];
   int len;
@@ -664,17 +666,17 @@ on_qth_name_changed                    (GtkEditable     *editable,
 }
 
 
-void
-on_locator_changed                     (GtkEditable     *editable,
-                                        gpointer         user_data)
+  void
+on_locator_changed(
+	GtkEditable *editable,
+	gpointer     user_data)
 {
   int len, idx;
   char buff[7];
 
   /* Get entry field, ignore blank field */
   Strlcpy( buff,
-	  gtk_entry_get_text(GTK_ENTRY(editable)),
-	  sizeof(buff) );
+	  gtk_entry_get_text(GTK_ENTRY(editable)), sizeof(buff) );
   len = (int)strlen(buff);
   if( len == 0 ) return;
 
@@ -727,9 +729,10 @@ on_locator_changed                     (GtkEditable     *editable,
 }
 
 
-void
-on_band_changed                        (GtkEditable     *editable,
-                                        gpointer         user_data)
+  void
+on_band_changed(
+	GtkEditable *editable,
+	gpointer     user_data)
 {
   int len, idx;
   char buff[14];
@@ -738,8 +741,7 @@ on_band_changed                        (GtkEditable     *editable,
 
   /* Get entry field, ignore blank field */
   Strlcpy( buff,
-	  gtk_entry_get_text(GTK_ENTRY(editable)),
-	  sizeof(buff) );
+	  gtk_entry_get_text(GTK_ENTRY(editable)), sizeof(buff) );
   len = (int)strlen(buff);
   if( len == 0 ) return;
 
@@ -759,9 +761,10 @@ on_band_changed                        (GtkEditable     *editable,
 }
 
 
-void
-on_callsign_changed                    (GtkEditable     *editable,
-                                        gpointer         user_data)
+  void
+on_callsign_changed(
+	GtkEditable *editable,
+	gpointer     user_data)
 {
   int len, idx, idy;
   char buff[15];
@@ -806,18 +809,17 @@ on_callsign_changed                    (GtkEditable     *editable,
 }
 
 
-void
-on_new_record_clicked                  (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_new_record_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   /* Variables for time and date */
   time_t tp;    /* Time type     */
   struct tm dt; /* Date and time */
 
   /* For reading Tcvr status */
-  char
-	freq[8],
-	rst[4];
+  char freq[8], rst[4];
 
   /* Save record if needed and completed */
   if( isFlagSet(SAVE_RECORD) )
@@ -841,9 +843,9 @@ on_new_record_clicked                  (GtkButton       *button,
 	GtkWidget *entry;
 
 	if( !Read_Rx_Status(freq, rst) ) return;
-	entry = lookup_widget( main_window, "rst_out" );
+	entry = Builder_Get_Object( main_window_builder, "rst_out" );
 	gtk_entry_set_text( GTK_ENTRY(entry), rst );
-	entry = lookup_widget( main_window, "band" );
+	entry = Builder_Get_Object( main_window_builder, "band" );
 	gtk_entry_set_text( GTK_ENTRY(entry), freq );
   }
 
@@ -859,11 +861,13 @@ on_new_record_clicked                  (GtkButton       *button,
 }
 
 
-void
-on_save_record_clicked                 (GtkButton       *button,
-										gpointer         user_data)
+  void
+on_save_record_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
-  /* Save record if needed (and valid) */
+  /* Save record if needed(
+	 and valid) */
   if( Read_QSO_Record() )
 	Save_Dialog( _("Save QSO Record to file?") );
   else Save_Dialog(
@@ -873,195 +877,264 @@ on_save_record_clicked                 (GtkButton       *button,
 
 
   void
-on_clear_fields_clicked                (GtkButton       *button,
-                                        gpointer         user_data)
+on_clear_fields_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Clear_Record_Fields(TRUE);
 }
 
 
-void
-on_clear_windows_clicked               (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_clear_windows_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Save_Pixbuf();
   SetFlag( CLEAR_TX_WINDOW | CLEAR_RX_WINDOW );
-  gtk_text_buffer_set_text(gbl_tx_text_buffer, "", -1);
+  gtk_text_buffer_set_text(tx_text_buffer, "", -1);
 }
 
 
-void
-on_macro1_clicked                      (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_macro1_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Select_Macro( 1 );
 }
 
 
-void
-on_macro2_clicked                      (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_macro2_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Select_Macro( 2 );
 }
 
 
-void
-on_macro3_clicked                      (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_macro3_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Select_Macro( 3 );
 }
 
 
-void
-on_macro4_clicked                      (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_macro4_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Select_Macro( 4 );
 }
 
 
-void
-on_macro5_clicked                      (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_macro5_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Select_Macro( 5 );
 }
 
 
-void
-on_macro6_clicked                      (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_macro6_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Select_Macro( 6 );
 }
 
 
-void
-on_macro7_clicked                      (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_macro7_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Select_Macro( 7 );
 }
 
 
-void
-on_macro8_clicked                      (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_macro8_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Select_Macro( 8 );
 }
 
 
-void
-on_macro9_clicked                      (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_macro9_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Select_Macro( 9 );
 }
 
 
-void
-on_macro10_clicked                     (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_macro10_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Select_Macro( 10 );
 }
 
 
-void
-on_error_dialog_destroy                (GtkObject       *object,
-                                        gpointer         user_data)
+  void
+on_error_dialog_destroy(
+	GObject  *object,
+	gpointer  user_data)
 {
   error_dialog = NULL;
 }
 
 
-gboolean
-on_error_dialog_delete_event           (GtkWidget       *widget,
-                                        GdkEvent        *event,
-                                        gpointer         user_data)
+  gboolean
+on_error_dialog_delete_event(
+	GtkWidget *widget,
+	GdkEvent  *event,
+	gpointer   user_data)
 {
-  return TRUE;
+  return( TRUE );
 }
 
 
-void
-on_error_quit_button_clicked           (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_error_quit_button_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Cleanup();
   gtk_main_quit();
 }
 
 
-void
-on_error_ok_button_clicked             (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_error_ok_button_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   gtk_widget_destroy( error_dialog );
 }
 
 
-void
-on_cancel_save_clicked                 (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_cancel_save_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   gtk_widget_destroy( save_dialog );
 }
 
 
-void
-on_save_ok_clicked                     (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_save_ok_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Save_QSO_Record();
   gtk_widget_destroy( save_dialog );
 }
 
 
-void
-on_feldhell_activate                   (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_feldhell_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   SetFlag( MODE_FELDHELL );
   New_Parameters();
 }
 
 
-void
-on_fmhell_activate                     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_fmhell_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   ClearFlag( MODE_FELDHELL );
   New_Parameters();
 }
 
 
-void
-on_bdrate_activate                     (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_bdrate_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
 	New_Parameters();
 }
 
 
-static GtkFileSelection *fileselection;
-void
-on_font_selector_activate              (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+static GtkFileChooser *filechooser = NULL;
+  void
+on_font_selector_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
-  fileselection = GTK_FILE_SELECTION(create_fileselection());
-  gtk_widget_show( GTK_WIDGET(fileselection) );
-  gtk_file_selection_set_filename( fileselection, "~/xfhell/fonts/" );
+  GtkBuilder *builder = NULL;
+  filechooser = GTK_FILE_CHOOSER( create_filechooser( &builder ) );
+  gtk_widget_show( GTK_WIDGET(filechooser) );
+  gtk_file_chooser_set_action( filechooser, GTK_FILE_CHOOSER_ACTION_OPEN );
+  char folder[128];
+  Strlcpy( folder, getenv("HOME"), sizeof(folder) );
+  Strlcat( folder, "/xfhell/fonts/", sizeof(folder) );
+  gtk_file_chooser_set_current_folder( filechooser, folder );
+  g_object_unref( builder );
 }
 
 
-void
-on_show_font_activate                  (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_filechooser_response(
+	GtkDialog *dialog,
+	gint       response_id,
+	gpointer   user_data)
+{
+  if( response_id == GTK_RESPONSE_OK )
+  {
+	if( strlen(gtk_file_chooser_get_filename(filechooser)) > 80 )
+	{
+	  Error_Dialog( _("Font file name to long"), QUIT );
+	  return;
+	}
+
+	Strlcpy( rc_data.font_file,
+		gtk_file_chooser_get_filename(filechooser),
+		sizeof(rc_data.font_file) );
+
+	if( strstr(rc_data.font_file, ".bdf") == NULL )
+	{
+	  Error_Dialog( _("This is not a .bdf font file"), QUIT );
+	  return;
+	}
+
+	New_Parameters();
+  } /* if( response_id == GTK_RESPONSE_OK ) */
+
+  gtk_widget_destroy( GTK_WIDGET(filechooser) );
+}
+
+
+  void
+on_filechooser_destroy(
+	GObject *object,
+	gpointer user_data)
+{
+  filechooser = NULL;
+}
+
+
+  void
+on_show_font_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   int idx;
 
@@ -1071,9 +1144,10 @@ on_show_font_activate                  (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_reverse_video_activate              (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_reverse_video_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
 	rc_data.rev_video = TRUE;
@@ -1082,9 +1156,10 @@ on_reverse_video_activate              (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_k2_activate                         (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_k2_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
   {
@@ -1096,9 +1171,10 @@ on_k2_activate                         (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_k3_activate                         (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_k3_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
   {
@@ -1110,9 +1186,10 @@ on_k3_activate                         (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_ft847_activate                      (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_ft847_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
   {
@@ -1124,9 +1201,10 @@ on_ft847_activate                      (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_ft857_activate                      (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_ft857_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
   {
@@ -1138,9 +1216,10 @@ on_ft857_activate                      (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_rts_ctl_activate                    (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_rts_ctl_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
   {
@@ -1152,9 +1231,10 @@ on_rts_ctl_activate                    (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_none_activate                       (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_none_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
   {
@@ -1166,36 +1246,40 @@ on_none_activate                       (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_md2x2_activate                      (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_md2x2_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   rc_data.dot_size = 2;
   New_Parameters();
 }
 
 
-void
-on_md3x3_activate                      (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_md3x3_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   rc_data.dot_size = 3;
   New_Parameters();
 }
 
 
-void
-on_md4x4_activate                      (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_md4x4_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   rc_data.dot_size = 4;
   New_Parameters();
 }
 
 
-void
-on_record_qsos_activate                (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_record_qsos_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
   {
@@ -1216,9 +1300,10 @@ on_record_qsos_activate                (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_capitalize_letters_activate         (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_capitalize_letters_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
 	SetFlag( CAPITALIZE );
@@ -1226,9 +1311,10 @@ on_capitalize_letters_activate         (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_loopback_tx_to_rx_activate          (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_loopback_tx_to_rx_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) )
 	SetFlag( TX2RX_LOOPBACK );
@@ -1236,9 +1322,10 @@ on_loopback_tx_to_rx_activate          (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_identify_in_cw_activate             (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_identify_in_cw_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   char mesg[MESG_SIZE];
 
@@ -1247,9 +1334,10 @@ on_identify_in_cw_activate             (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_quit_activate                       (GtkMenuItem     *menuitem,
-                                        gpointer         user_data)
+  void
+on_quit_activate(
+	GtkMenuItem *menuitem,
+	gpointer     user_data)
 {
   if( isFlagSet(TRANSMIT_MODE) ||
 	  isFlagSet(RECEIVE_MODE) )
@@ -1264,77 +1352,31 @@ on_quit_activate                       (GtkMenuItem     *menuitem,
 }
 
 
-void
-on_bad_entry_ok_clicked                (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_entry_ok_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
-  gtk_widget_destroy( bad_entry_dialog );
+  gtk_widget_destroy( entry_dialog );
 }
 
 
-void
-on_cancel_quit_clicked                 (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_cancel_quit_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   gtk_widget_destroy( quit_dialog );
   quit_dialog  = NULL;
 }
 
 
-void
-on_quit_ok_clicked                     (GtkButton       *button,
-                                        gpointer         user_data)
+  void
+on_quit_ok_clicked(
+	GtkButton *button,
+	gpointer   user_data)
 {
   Cleanup();
   gtk_main_quit();
 }
 
-
-void
-on_fileselection_response              (GtkDialog       *dialog,
-                                        gint             response_id,
-                                        gpointer         user_data)
-{
-  if( response_id == GTK_RESPONSE_OK )
-  {
-	if( strlen(gtk_file_selection_get_filename(fileselection)) > 80 )
-	{
-	  Error_Dialog( _("Font file name to long"), QUIT );
-	  return;
-	}
-
-	Strlcpy( rc_data.font_file,
-		gtk_file_selection_get_filename(fileselection),
-		sizeof(rc_data.font_file) );
-	if( strstr(rc_data.font_file, ".bdf") == NULL )
-	{
-	  Error_Dialog( _("This is not a .bdf font file"), QUIT );
-	  return;
-	}
-	New_Parameters();
-	gtk_widget_destroy( GTK_WIDGET(fileselection) );
-	return;
-
-  } /* if( response_id == GTK_RESPONSE_OK ) */
-
-  gtk_widget_destroy( GTK_WIDGET(fileselection) );
-}
-
-
-void
-on_scope_button_clicked                (GtkButton       *button,
-										gpointer         user_data)
-{
-  if( isFlagSet( ENABLE_SCOPE ) )
-  {
-	gtk_label_set_text( GTK_LABEL(gbl_scope_label), "Waterfall" );
-	ClearFlag( ENABLE_SCOPE );
-  }
-  else
-  {
-	gtk_label_set_text( GTK_LABEL(gbl_scope_label), _("Signal Level") );
-	SetFlag( ENABLE_SCOPE );
-  }
-  gtk_widget_queue_draw( gbl_scope );
-}
-
