@@ -1,13 +1,4 @@
-/*  cat.c
- *
- *  CAT control functions for the xfhell application
- */
-
 /*
- *  xfhell: An application to transmit and receive
- *  Feld Hell signals using a computer's sound card
- *
- *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
  *  published by the Free Software Foundation; either version 3 of
@@ -67,6 +58,185 @@ static char
 
 /*------------------------------------------------------------------------*/
 
+/* Cancel_CAT()
+ *
+ * Cancels Transceiver CAT (on error)
+ */
+
+  static void
+Cancel_CAT( void )
+{
+  close( serial_fd );
+  serial_fd = 0;
+  rc_data.tcvr_type = NONE;
+  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
+		Builder_Get_Object(popup_menu_builder, "none")), TRUE );
+  ClearFlag( CAT_SETUP );
+  ClearFlag( TCVR_SERIAL_TEST );
+  Error_Dialog( "Failed to Establish Transceiver CAT", OK  );
+}
+
+/*-------------------------------------------------------------------------*/
+
+/*  Read_Tcvr_Serial()
+ *
+ *  Reading Data from the Tcvr's Serial Port
+ */
+
+  static void
+Read_Tcvr_Serial( char *data, size_t len )
+{
+  int nbytes = 0, cnt = 0;
+
+  /* Check for data to be available */
+  if( isFlagClear(TCVR_SERIAL_TEST) &&
+	  isFlagClear(CAT_SETUP) )
+	return;
+  while( ++cnt < MAX_SERIAL_RETRIES )
+  {
+	ioctl( serial_fd, FIONREAD, &nbytes);
+	if( nbytes == (int)len ) break;
+	fprintf( stderr,
+		"xfhell: read(): incorrect number of bytes available:"
+		" %d/%d\n", nbytes, (int)len );
+	usleep(100000);
+  }
+
+  /* Error condition */
+  if( cnt >= MAX_SERIAL_RETRIES )
+  {
+	tcflush( serial_fd, TCIOFLUSH );
+	fprintf( stderr,
+		"xfhell: read(): incorrect number of bytes available:"
+		" %d/%d after %d tries\n", nbytes, (int)len, cnt );
+	data[0] = '\0';
+	Cancel_CAT();
+	return;
+  }
+
+  /* Clear data buffer and read from serial */
+  size_t ret = (size_t)read( serial_fd, data, len );
+
+  /* Error condition */
+  if( ret != len )
+  {
+	tcflush( serial_fd, TCIOFLUSH );
+	fprintf( stderr,
+		"xfhell: read() return value wrong: %d/%d\n",
+		(int)ret, (int)len );
+	data[0] = '\0';
+	Cancel_CAT();
+	return;
+  }
+
+  return;
+} /* End of Read_Tcvr_Serial() */
+
+/*-------------------------------------------------------------------------*/
+
+/*  Write_Tcvr_Serial()
+ *
+ *  Writes a command to the Tranceiver
+ */
+
+  static void
+Write_Tcvr_Serial( char *data, size_t len )
+{
+  int cnt = 0;
+  size_t ret = 0;
+
+  /* Flush serial and write command, retrying if needed */
+  if( isFlagClear(TCVR_SERIAL_TEST) &&
+	  isFlagClear(CAT_SETUP) )
+	return;
+
+  tcflush( serial_fd, TCIOFLUSH );
+  while( cnt++ < MAX_SERIAL_RETRIES )
+  {
+	ret = (size_t)write( serial_fd, data, len );
+	if( ret == len ) break;
+	fprintf( stderr,
+		"xfhell: write(): incorrect number of bytes written:"
+		" %d/%d\n", (int)ret, (int)len );
+	usleep(100000);
+  }
+
+  /* Error condition */
+   if( cnt >= MAX_SERIAL_RETRIES )
+  {
+	tcflush( serial_fd, TCIOFLUSH );
+	fprintf( stderr,
+		"xfhell: write(): incorrect number of bytes written:"
+		" %d/%d after %d tries\n", (int)ret, (int)len, cnt );
+	Cancel_CAT();
+	return;
+  }
+
+  /* Give time to CAT to do its thing */
+  usleep(100000);
+  return;
+
+} /* End of Write_Tcvr_Command() */
+
+/*-------------------------------------------------------------------------*/
+
+/* Tcvr_Status()
+ *
+ * Gets or Sets Transceiver status (only for the K2/K3 currently)
+ * Gets or Sets the Tx Power O/P, Mic Gain and Compressor level
+ */
+  static gboolean
+Tcvr_Status( int action, tcvr_status_t *status )
+{
+  /* For K2 or K3 */
+  if( (rc_data.tcvr_type == K2) || (rc_data.tcvr_type == K3) )
+  {
+	/* Get transceiver status */
+	if( action == GET_TCVR_STATUS )
+	{
+	  Write_Tcvr_Serial( K3_POWER_G, sizeof(K3_POWER_G)-1 );
+	  Read_Tcvr_Serial(  K3_POWER_S, sizeof(K3_POWER_S)-1 );
+	  if( rc_data.tcvr_type == K3 )
+	  {
+		Write_Tcvr_Serial( K3_COMP_G, sizeof(K3_COMP_G)-1 );
+		Read_Tcvr_Serial(  K3_COMP_S, sizeof(K3_COMP_S)-1 );
+	  }
+	  Write_Tcvr_Serial( K3_VFOA_G, sizeof(K3_VFOA_G)-1 );
+	  Read_Tcvr_Serial(  K3_VFOA_S, sizeof(K3_VFOA_S)-1 );
+	  Write_Tcvr_Serial( K3_MODE_G, sizeof(K3_MODE_G)-1 );
+	  Read_Tcvr_Serial(  K3_MODE_S, sizeof(K3_MODE_S)-1 );
+	  status->tx_power = atoi( K3_POWER_S+2 );
+	  status->comp_lev = atoi(  K3_COMP_S+2 );
+	  status->rig_mode = atoi(  K3_MODE_S+2 );
+	  status->vfo_freq = atoi(  K3_VFOA_S+2 );
+	  return( TRUE );
+	} /* if( action == GET_TCVR_STATUS ) */
+
+	/* Set transceiver status */
+	if( action == SET_TCVR_STATUS )
+	{
+	  snprintf( K3_POWER_S+2, sizeof(K3_POWER_S)-2,  "%03d;", status->tx_power );
+	  snprintf(  K3_COMP_S+2, sizeof(K3_COMP_S)-2,  "%03d;", status->comp_lev );
+	  snprintf(  K3_MODE_S+2, sizeof(K3_MODE_S)-2,    "%d;", status->rig_mode );
+	  snprintf(  K3_VFOA_S+2, sizeof(K3_VFOA_S)-2, "%011d;", status->vfo_freq );
+
+	  Write_Tcvr_Serial( K3_VFOA_S, sizeof(K3_VFOA_S)-1 );
+	  sleep( 1 );
+	  Write_Tcvr_Serial( K3_MODE_S, sizeof(K3_MODE_S)-1 );
+	  Write_Tcvr_Serial( K3_VFOA_S, sizeof(K3_VFOA_S)-1 );
+	  if( rc_data.tcvr_type == K3 )
+		Write_Tcvr_Serial( K3_COMP_S, sizeof(K3_COMP_S)-1 );
+	  Write_Tcvr_Serial(K3_POWER_S, sizeof(K3_POWER_S)-1);
+	  return( TRUE );
+
+	} /* if( action == SET_TCVR_STATUS ) */
+  } /* if( (rc_data.tcvr_type == K2) || (rc_data.tcvr_type == K2) ) */
+
+  return( FALSE );
+} /* Tcvr_Status() */
+
+/*-------------------------------------------------------------------------*/
+
 /*  Open_Tcvr_Serial()
  *
  *  Opens Tcvr's Serial Port device, returns the file
@@ -94,7 +264,7 @@ Open_Tcvr_Serial( void )
 	perror( rc_data.cat_serial );
 	rc_data.tcvr_type = NONE;
 	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
-		  lookup_widget(gbl_popup_menu, "none")), TRUE );
+		  Builder_Get_Object(popup_menu_builder, "none")), TRUE );
 	snprintf( mesg, sizeof(mesg),
 		_("Failed to open %s\n"\
 		  "CAT not enabled"), rc_data.cat_serial );
@@ -117,9 +287,9 @@ Open_Tcvr_Serial( void )
 	perror( rc_data.cat_serial );
 	rc_data.tcvr_type = NONE;
 	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
-		  lookup_widget(gbl_popup_menu, "none")), TRUE );
+		  Builder_Get_Object(popup_menu_builder, "none")), TRUE );
 	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
-		  lookup_widget(gbl_popup_menu, "none")), TRUE );
+		  Builder_Get_Object(popup_menu_builder, "none")), TRUE );
 	if( fcntl(serial_fd, F_GETLK, &lockinfo) != -1 )
 	{
 	  snprintf( mesg, sizeof(mesg),
@@ -173,7 +343,7 @@ Open_Tcvr_Serial( void )
 	}
 	rc_data.tcvr_type = NONE;
 	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
-		  lookup_widget(gbl_popup_menu, "none")), TRUE );
+		  Builder_Get_Object(popup_menu_builder, "none")), TRUE );
 	Error_Dialog( _("Failed to Enable CAT"), OK );
 	return( FALSE );
   }
@@ -183,41 +353,23 @@ Open_Tcvr_Serial( void )
   switch( rc_data.tcvr_type )
   {
 	case FT847:
-	  if( !Write_Tcvr_Serial(FT847_CAT_ON, sizeof(FT847_CAT_ON)) ||
-		  !Write_Tcvr_Serial(FT847_MAIN_VFO, sizeof(FT847_MAIN_VFO)) ||
-		  !Read_Tcvr_Serial(test, 5) )
-	  {
-		fprintf( stderr, "xfhell: failed to enable FT847 CAT\n" );
-		Write_Tcvr_Serial( FT847_CAT_OFF, sizeof(FT847_CAT_OFF) );
-		Cancel_CAT();
-		return( FALSE );
-	  }
+	  Write_Tcvr_Serial( FT847_CAT_ON, sizeof(FT847_CAT_ON) );
+	  Write_Tcvr_Serial( FT847_MAIN_VFO, sizeof(FT847_MAIN_VFO) );
+	  Read_Tcvr_Serial( test, 5 );
 	  break;
 
 	case FT857:
-	  if( !Write_Tcvr_Serial(FT847_MAIN_VFO, sizeof(FT847_MAIN_VFO)) ||
-		  !Read_Tcvr_Serial(test, 5) )
-	  {
-		fprintf( stderr, "xfhell: failed to enable FT857 CAT\n" );
-		Cancel_CAT();
-		return( FALSE );
-	  }
+	  Write_Tcvr_Serial( FT847_MAIN_VFO, sizeof(FT847_MAIN_VFO) );
+	  Read_Tcvr_Serial( test, 5);
 	  break;
 
-
 	case K2:
 	  /* Get K2 VFO A as a test of serial port */
-	  if( !Write_Tcvr_Serial(K3_VFOA_G, sizeof(K3_VFOA_G)-1) ||
-		  !Read_Tcvr_Serial(K3_VFOA_S, sizeof(K3_VFOA_S)-1) )
-	  {
-		fprintf( stderr, "xfhell: failed to enable K2 CAT\n" );
-		Cancel_CAT();
-		return( FALSE );
-	  }
+	  Write_Tcvr_Serial( K3_VFOA_G, sizeof(K3_VFOA_G)-1 );
+	  Read_Tcvr_Serial(  K3_VFOA_S, sizeof(K3_VFOA_S)-1 );
 
 	  /* Get original tcvr status */
-	  if( !Tcvr_Status(GET_TCVR_STATUS, &orig_tcvr_status) )
-		return( FALSE );
+	  Tcvr_Status( GET_TCVR_STATUS, &orig_tcvr_status );
 
 	  /* Set tcvr power and compressor level */
 	  if( !Set_Tx_Power(rc_data.tx_output, 0) )
@@ -227,15 +379,10 @@ Open_Tcvr_Serial( void )
 	case K3:
 	  /* Set K3 to mode K31 as a test of serial port */
 	  Strlcpy( K3_K31_S, "K31;", sizeof(K3_K31_S) );
-	  if( !Write_Tcvr_Serial(K3_K31_S, sizeof(K3_K31_S)-1) )
-	  {
-		fprintf( stderr, "xfhell: failed to enable K3 CAT\n" );
-		Cancel_CAT();
-		return( FALSE );
-	  }
-	  if( !Write_Tcvr_Serial(K3_K31_G, sizeof(K3_K31_G)-1) ||
-		  !Read_Tcvr_Serial(K3_K31_S, sizeof(K3_K31_S)-1) ||
-		  (strcmp(K3_K31_S, "K31;") != 0) )
+	  Write_Tcvr_Serial( K3_K31_S, sizeof(K3_K31_S)-1 );
+	  Write_Tcvr_Serial( K3_K31_G, sizeof(K3_K31_G)-1 );
+	  Read_Tcvr_Serial(  K3_K31_S, sizeof(K3_K31_S)-1 );
+	  if( strcmp(K3_K31_S, "K31;") != 0 )
 	  {
 		fprintf( stderr, "xfhell: failed to enable K3 CAT\n" );
 		Cancel_CAT();
@@ -243,8 +390,7 @@ Open_Tcvr_Serial( void )
 	  }
 
 	  /* Get original tcvr status */
-	  if( !Tcvr_Status(GET_TCVR_STATUS, &orig_tcvr_status) )
-		return( FALSE );
+	  Tcvr_Status( GET_TCVR_STATUS, &orig_tcvr_status );
 
 	  /* Set tcvr power and compressor level */
 	  if( !Set_Tx_Power(rc_data.tx_output, 0) )
@@ -261,6 +407,57 @@ Open_Tcvr_Serial( void )
 
 /*-------------------------------------------------------------------------*/
 
+/* RS232_RTS_DTR_Set()
+ *
+ * Sets the Serial Port RTS/DTR line UP/DOWN
+ */
+  static gboolean
+RS232_RTS_DTR_Set( gboolean set )
+{
+  int modem_lines;
+
+  /* Abort if CAT disabled */
+  if( isFlagClear(RTS_DTR_SETUP) ) return( TRUE );
+
+  /* Read current modem state */
+  if( ioctl(serial_fd, TIOCMGET, &modem_lines) == -1 )
+  {
+	perror( rc_data.rts_serial );
+	rc_data.tcvr_type = NONE;
+	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
+		  Builder_Get_Object(popup_menu_builder, "none")), TRUE );
+	Error_Dialog(
+		_("Failed to set RTS/DTR\n"\
+		  "RTS/DTR Control disabled"), OK );
+	ClearFlag( RTS_DTR_SETUP );
+	return( FALSE );
+  }
+
+  /* Change the RTS/DTR state */
+  if( set )
+	modem_lines |=  (TIOCM_RTS | TIOCM_DTR);
+  else
+	modem_lines &= ~(TIOCM_RTS | TIOCM_DTR);
+
+  /* Write the new setting */
+  if( ioctl(serial_fd, TIOCMSET, &modem_lines) == -1 )
+  {
+	perror( rc_data.rts_serial );
+	rc_data.tcvr_type = NONE;
+	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
+		  Builder_Get_Object(popup_menu_builder, "none")), TRUE );
+	Error_Dialog(
+		_("Failed to set RTS/DTR\n"\
+		  "RTS/DTR Control disabled"), OK );
+	ClearFlag( RTS_DTR_SETUP );
+	return( FALSE );
+  }
+
+  return( TRUE );
+} /* RS232_RTS_DTR_Set() */
+
+/*-------------------------------------------------------------------------*/
+
 /*  Close_Tcvr_Serial()
  *
  *  Restore old options and close the Serial port
@@ -306,6 +503,7 @@ Close_Tcvr_Serial( void )
 	ClearFlag( RTS_DTR_SETUP );
 
   } /* if( isFlagSet(CAT_SETUP) || isFlagSet(RTS_DTR_SETUP) ) */
+
 } /* End of Close_Tcvr_Serial() */
 
 /*-------------------------------------------------------------------------*/
@@ -333,7 +531,7 @@ Open_RTS_DTR_Serial( void )
 	perror( rc_data.rts_serial );
 	rc_data.tcvr_type = NONE;
 	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
-		  lookup_widget(gbl_popup_menu, "none")), TRUE );
+		  Builder_Get_Object(popup_menu_builder, "none")), TRUE );
 	snprintf( mesg, sizeof(mesg),
 		_("Failed to open %s\n"\
 		  "RTS/DTR Control not enabled"), rc_data.rts_serial );
@@ -356,7 +554,7 @@ Open_RTS_DTR_Serial( void )
 	perror( rc_data.rts_serial );
 	rc_data.tcvr_type = NONE;
 	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
-		  lookup_widget(gbl_popup_menu, "none")), TRUE );
+		  Builder_Get_Object(popup_menu_builder, "none")), TRUE );
 	if( fcntl(serial_fd, F_GETLK, &lockinfo) == 0 )
 	{
 	  snprintf( mesg, sizeof(mesg),
@@ -392,7 +590,7 @@ Open_RTS_DTR_Serial( void )
 	}
 	rc_data.tcvr_type = NONE;
 	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
-		  lookup_widget(gbl_popup_menu, "none")), TRUE );
+		  Builder_Get_Object(popup_menu_builder, "none")), TRUE );
 	Error_Dialog( _("Failed to Enable RTS/DTR Control"), OK );
 	return( FALSE );
   }
@@ -403,126 +601,105 @@ Open_RTS_DTR_Serial( void )
 
 /*-------------------------------------------------------------------------*/
 
-
-/* Cancel_CAT()
+/*  Read_Rx_Freq()
  *
- * Cancels Transceiver CAT (on error)
+ *  Reads the Rx freq of the Yaesu transceiver.
  */
 
-  void
-Cancel_CAT( void )
+  static gboolean
+Read_Rx_Freq( int *freq )
 {
-  close( serial_fd );
-  serial_fd = 0;
-  rc_data.tcvr_type = NONE;
-  gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
-		lookup_widget(gbl_popup_menu, "none")), TRUE );
-  ClearFlag( CAT_SETUP );
-  ClearFlag( TCVR_SERIAL_TEST );
-  Error_Dialog( "Failed to Establish Transceiver CAT", OK  );
-}
+  /* Abort if CAT disabled */
+  if( isFlagClear(CAT_SETUP) ) return( TRUE );
 
-/*-------------------------------------------------------------------------*/
+  /* A char string for sending and receiving  */
+  /* data strings to and from the transceiver */
+  char cmnd_parm[5];
 
-/*  Read_Tcvr_Serial()
- *
- *  Reading Data from the Tcvr's Serial Port
- */
-
-  gboolean
-Read_Tcvr_Serial( char *data, size_t len )
-{
-  int nbytes = 0, cnt = 0;
-
-  /* Check for data to be available */
-  if( isFlagClear(TCVR_SERIAL_TEST) &&
-	  isFlagClear(CAT_SETUP) )
-	return( FALSE );
-  while( ++cnt < MAX_SERIAL_RETRIES )
+  switch( rc_data.tcvr_type )
   {
-	ioctl( serial_fd, FIONREAD, &nbytes);
-	if( nbytes == (int)len ) break;
-	fprintf( stderr,
-		"xfhell: read(): incorrect number of bytes available:"
-		" %d/%d\n", nbytes, (int)len );
-	usleep(100000);
-  }
+	case FT847: case FT857:
+	  /* Read Rx frequency */
+	  bzero( cmnd_parm, sizeof(cmnd_parm) );
+	  Write_Tcvr_Serial( FT847_MAIN_VFO, sizeof(FT847_MAIN_VFO) );
+	  Read_Tcvr_Serial( cmnd_parm, sizeof(cmnd_parm) );
 
-  /* Error condition */
-  if( cnt >= MAX_SERIAL_RETRIES )
-  {
-	tcflush( serial_fd, TCIOFLUSH );
-	fprintf( stderr,
-		"xfhell: read(): incorrect number of bytes available:"
-		" %d/%d after %d tries\n", nbytes, (int)len, cnt );
-	data[0] = '\0';
-	Cancel_CAT();
-	return( FALSE );
-  }
+	  /* Decode frequency data to 10Hz */
+	  *freq = 0;
+	  *freq += (cmnd_parm[0] & 0xF0) >> 4;
+	  *freq *= 10;
+	  *freq += cmnd_parm[0] & 0x0F;
+	  *freq *= 10;
+	  *freq += (cmnd_parm[1] & 0xF0) >> 4;
+	  *freq *= 10;
+	  *freq += cmnd_parm[1] & 0x0F;
+	  *freq *= 10;
+	  *freq += (cmnd_parm[2] & 0xF0) >> 4;
+	  *freq *= 10;
+	  *freq += cmnd_parm[2] & 0x0F;
+	  *freq *= 10;
+	  *freq += (cmnd_parm[3] & 0xF0) >> 4;
+	  *freq *= 10;
+	  *freq += cmnd_parm[3] & 0x0F;
+	  *freq *= 10;
+	  break;
 
-  /* Clear data buffer and read from serial */
-  size_t ret = (size_t)read( serial_fd, data, len );
+	case K2: case K3:
+	  /* Read Rx frequency */
+	  Write_Tcvr_Serial( K3_VFOA_G, sizeof(K3_VFOA_G)-1 );
+	  Read_Tcvr_Serial(  K3_VFOA_S, sizeof(K3_VFOA_S)-1 );
+	  *freq = atoi( K3_VFOA_S+2 );
+	  break;
 
-  /* Error condition */
-  if( ret != len )
-  {
-	tcflush( serial_fd, TCIOFLUSH );
-	fprintf( stderr,
-		"xfhell: read() return value wrong: %d/%d\n",
-		(int)ret, (int)len );
-	data[0] = '\0';
-	Cancel_CAT();
-	return( FALSE );
-  }
+  } /* switch( rc_data.tcvr_type ) */
 
   return( TRUE );
-} /* End of Read_Tcvr_Serial() */
+} /* End of Read_Rx_Freq() */
 
-/*-------------------------------------------------------------------------*/
+/*------------------------------------------------------------------------*/
 
-/*  Write_Tcvr_Serial()
+/*  Write_Rx_Freq()
  *
- *  Writes a command to the Tranceiver
+ *  Writes the Rx freq to the Yaesu transceiver.
  */
 
-  gboolean
-Write_Tcvr_Serial( char *data, size_t len )
+  static gboolean
+Write_Rx_Freq( int freq )
 {
-  int cnt = 0;
-  size_t ret = 0;
+  /* Buffer used for converting freq. to string */
+  char freq_buff[9];
+  int idx; /* Index for loops etc */
 
-  /* Flush serial and write command, retrying if needed */
-  if( isFlagClear(TCVR_SERIAL_TEST) &&
-	  isFlagClear(CAT_SETUP) )
-	return( FALSE );
 
-  tcflush( serial_fd, TCIOFLUSH );
-  while( cnt++ < MAX_SERIAL_RETRIES )
+  /* Abort if CAT disabled */
+  if( isFlagClear(CAT_SETUP) ) return( TRUE );
+
+  switch( rc_data.tcvr_type )
   {
-	ret = (size_t)write( serial_fd, data, len );
-	if( ret == len ) break;
-	fprintf( stderr,
-		"xfhell: write(): incorrect number of bytes written:"
-		" %d/%d\n", (int)ret, (int)len );
-	usleep(100000);
-  }
+	case FT847: case FT857:
+	  /* Set Rx frequency */
+	  bzero( FT847_SET_VFO, 4 );
+	  if( freq > 470000000 ) freq = 470000000;
+	  if( freq < 0 ) freq = 0;
+	  snprintf( freq_buff, sizeof(freq_buff), "%08d", freq / 10 );
+	  for( idx = 0; idx < 4; idx++ )
+	  {
+		FT847_SET_VFO[idx]  = (char)((freq_buff[2*idx]   - '0') << 4);
+		FT847_SET_VFO[idx] |= (freq_buff[2*idx+1] - '0');
+	  }
+	  Write_Tcvr_Serial( FT847_SET_VFO, sizeof(FT847_SET_VFO) );
+	  break;
 
-  /* Error condition */
-   if( cnt >= MAX_SERIAL_RETRIES )
-  {
-	tcflush( serial_fd, TCIOFLUSH );
-	fprintf( stderr,
-		"xfhell: write(): incorrect number of bytes written:"
-		" %d/%d after %d tries\n", (int)ret, (int)len, cnt );
-	Cancel_CAT();
-	return( FALSE );
-  }
+	case K2: case K3:
+	  /* Set Rx frequency */
+	  snprintf( K3_VFOA_S+2, sizeof(K3_VFOA_S)-2, "%011d;", freq );
+	  Write_Tcvr_Serial( K3_VFOA_S, sizeof(K3_VFOA_S)-1 );
+	  break;
+
+  } /* switch( rc_data.tcvr_type ) */
 
-  /* Give time to CAT to do its thing */
-  usleep(100000);
   return( TRUE );
-
-} /* End of Write_Tcvr_Command() */
+} /* Write_Rx_Freq() */
 
 /*-------------------------------------------------------------------------*/
 
@@ -550,9 +727,8 @@ Read_Rx_Status( char *freq, char *rst )
 	case FT847: case FT857:
 	  /* Read Rx status (S-meter) */
 	  bzero( tcvr_status, sizeof(tcvr_status) );
-	  if( !Write_Tcvr_Serial(FT847_RX_STATUS, sizeof(FT847_RX_STATUS)) ||
-		  !Read_Tcvr_Serial(tcvr_status, 1) )
-		return( FALSE );
+	  Write_Tcvr_Serial( FT847_RX_STATUS, sizeof(FT847_RX_STATUS) );
+	  Read_Tcvr_Serial(  tcvr_status, 1 );
 
 	  /* Mask out S-meter */
 	  if( rc_data.tcvr_type == FT847 )
@@ -570,9 +746,8 @@ Read_Rx_Status( char *freq, char *rst )
 	  if( smt > 9 ) smt = 9;
 
 	  /* Read Rx mode and frequency */
-	  if( !Write_Tcvr_Serial(FT847_MAIN_VFO, sizeof(FT847_MAIN_VFO)) ||
-		  !Read_Tcvr_Serial(tcvr_status, sizeof(tcvr_status)) )
-		return( FALSE );
+	  Write_Tcvr_Serial( FT847_MAIN_VFO, sizeof(FT847_MAIN_VFO) );
+	  Read_Tcvr_Serial(  tcvr_status, sizeof(tcvr_status) );
 
 	  /* Decode frequency data to 1 kHz */
 	  frq = 0;
@@ -591,14 +766,13 @@ Read_Rx_Status( char *freq, char *rst )
 
 	case K2: case K3:
 	  /* Read Rx frequency */
-	  if( !Read_Rx_Freq(&frq) ) return( FALSE );
+	  Read_Rx_Freq( &frq );
 	  frq += 500;
 	  frq /= 1000;
 
 	  /* Read S-meter */
-	  if( !Write_Tcvr_Serial(K3_SMTR_G, sizeof(K3_SMTR_G)-1) ||
-		  !Read_Tcvr_Serial(K3_SMTR_S, sizeof(K3_SMTR_S)-1) )
-		return( FALSE );
+	  Write_Tcvr_Serial( K3_SMTR_G, sizeof(K3_SMTR_G)-1 );
+	  Read_Tcvr_Serial(  K3_SMTR_S, sizeof(K3_SMTR_S)-1 );
 
 	  /* Following makes S-meter reading
 	   * more like the amateur RST code */
@@ -618,9 +792,10 @@ Read_Rx_Status( char *freq, char *rst )
   } /* switch( rc_data.tcvr_type ) */
 
   /* Enter frequency and S-meter data */
-  snprintf( freq, 8, "%-7.3f", (float)frq/1000. );
+  snprintf( freq, 8, "%-7.3f", (double)frq/1000.0 );
   if( smt > 9 ) smt = 9;
   smt = 500 + 10 * smt + 9;
+  if( smt < 0 ) smt = 0;
   snprintf( rst, 4, "%3d", smt );
 
   return( TRUE );
@@ -648,19 +823,18 @@ Set_Tcvr_Mode( void )
 	  switch( rc_data.tcvr_type )
 	  {
 		case FT847: case FT857:
-		  if( !Write_Tcvr_Serial( FT847_MODE_USB, sizeof(FT847_MODE_USB) ) ||
-			  !Write_Tcvr_Serial( FT847_PTT_ON, sizeof(FT847_PTT_ON) ) )
-			return( FALSE );
+		  Write_Tcvr_Serial( FT847_MODE_USB, sizeof(FT847_MODE_USB) );
+		  Write_Tcvr_Serial( FT847_PTT_ON, sizeof(FT847_PTT_ON) );
 		  break;
 
 		case K2: case K3:
-		  if( !Write_Tcvr_Serial( K3_MODE_USB, sizeof(K3_MODE_USB)-1 ) ||
-			  !Write_Tcvr_Serial( K3_PTT_ON, sizeof(K3_PTT_ON)-1 ) )
-			return( FALSE );
+		  Write_Tcvr_Serial( K3_MODE_USB, sizeof(K3_MODE_USB)-1 );
+		  Write_Tcvr_Serial( K3_PTT_ON, sizeof(K3_PTT_ON)-1 );
 		  break;
 
 	  } /* switch( rc_data.tcvr_type ) */
-	  sleep(1);
+
+	  sleep( 1 );
 	}/*  if( isFlagSet(TRANSMIT_MODE) ) */
 
 	if( isFlagSet(RECEIVE_MODE) )
@@ -668,21 +842,18 @@ Set_Tcvr_Mode( void )
 	  switch( rc_data.tcvr_type )
 	  {
 		case FT847:
-		  if( !Write_Tcvr_Serial( FT847_PTT_OFF, sizeof(FT847_PTT_OFF) ) ||
-			  !Write_Tcvr_Serial( FT847_MODE_CWN, sizeof(FT847_MODE_CWN) ) )
-			return( FALSE );
+		  Write_Tcvr_Serial( FT847_PTT_OFF, sizeof(FT847_PTT_OFF) );
+		  Write_Tcvr_Serial( FT847_MODE_CWN, sizeof(FT847_MODE_CWN) );
 		  break;
 
 		case FT857:
-		  if( !Write_Tcvr_Serial( FT847_PTT_OFF, sizeof(FT847_PTT_OFF) ) ||
-			  !Write_Tcvr_Serial( FT847_MODE_CW, sizeof(FT847_MODE_CW) ) )
-			return( FALSE );
+		  Write_Tcvr_Serial( FT847_PTT_OFF, sizeof(FT847_PTT_OFF) );
+		  Write_Tcvr_Serial( FT847_MODE_CW, sizeof(FT847_MODE_CW) );
 		  break;
 
 		case K2: case K3:
-		  if( !Write_Tcvr_Serial( K3_PTT_OFF, sizeof(K3_PTT_OFF)-1 ) ||
-			  !Write_Tcvr_Serial( K3_MODE_CWR, sizeof(K3_MODE_CWR)-1 ) )
-			return( FALSE );
+		  Write_Tcvr_Serial( K3_PTT_OFF, sizeof(K3_PTT_OFF)-1 );
+		  Write_Tcvr_Serial( K3_MODE_CWR, sizeof(K3_MODE_CWR)-1 );
 		  break;
 
 	  } /* switch( rc_data.tcvr_type ) */
@@ -720,8 +891,8 @@ Tune_Tcvr( double x )
   int
 	from, to,	/* Range to scan for a max bin value  */
 	bin_max,	/* Max bin value found in this range  */
-	max_idx,	/* dft idx at which the max is found */
-	dft_idx,	/* Idx used to search dft bin values */
+	max_idx,	/* ifft idx at which the max is found */
+	ifft_idx,	/* Idx used to search ifft bin values */
 	tcvr_freq,	/* Transceiver's Rx frequency */
 	audio_freq;	/* Audio frequency in waterfal window */
 
@@ -732,32 +903,42 @@ Tune_Tcvr( double x )
   /* Abort if CAT disabled */
   if( isFlagClear(CAT_SETUP) ) return( TRUE );
 
-  /* Calculate the dft index corresponding to
-   * pointer's x coordinate in waterfall window */
-  dft_idx = (int)(x - 0.5);
+  /* Calculate ifft index corresponding to pointer x
+   * in waterfall window. This is +1 over the pointer
+   * position since the first FFT bin is not used */
+  ifft_idx = (int)(x + 1.5);
 
   /* Look above and below click point for max bin val */
-  from = dft_idx - 10;
-  if( from < 0 )
-	from = 0;
-  to = dft_idx + 10;
-  if( to >= wfall_pixbuf.width ) to = wfall_pixbuf.width - 1;
+  from = ifft_idx - CLICK_TUNE_RANGE;
+  if( from < 0 ) from = 0;
+  to = ifft_idx + CLICK_TUNE_RANGE;
+  if( to >= wfall_pixbuf.width )
+	to = wfall_pixbuf.width - 1;
 
   /* Find max bin value around click point */
   bin_max = 0;
-  max_idx = dft_idx;
-  for( dft_idx = from; dft_idx < to; dft_idx++ )
-	if( bin_max < gbl_bin_ave[dft_idx] )
+  max_idx = ifft_idx;
+  for( ifft_idx = from; ifft_idx < to; ifft_idx++ )
+	if( bin_max < bin_ave[ifft_idx] )
 	{
-	  bin_max = gbl_bin_ave[dft_idx];
-	  max_idx = dft_idx;
+	  bin_max = bin_ave[ifft_idx];
+	  max_idx = ifft_idx;
 	}
 
-  /* Audio freq. corresponding to dft index */
-  audio_freq = (2 * max_idx * rc_data.tone_freq) / wfall_pixbuf.width;
+  /* Audio frequency corresponding to ifft index. Since the
+   * waterfall width has to be an odd number so that a center
+   * line may be present, and since the FFT has to be a power
+   * of 2 (e.g. even), the first (DC) output bin of the FFT is
+   * not used and hence max_idx has to be increased by 1 and
+   * the waterfall width also increased by one to correspond
+   * with the FFT width. Only this way the audio frequency that
+   * corresponds to FFT index can be calculated accurately. */
+  audio_freq =
+	( 2 * (max_idx + 1) * rc_data.tone_freq ) / ( wfall_pixbuf.width + 1 );
 
   /* Read current Rx frequency */
-  if( !Read_Rx_Freq(&tcvr_freq) ) return( FALSE );
+  tcvr_freq = 0;
+  Read_Rx_Freq( &tcvr_freq );
   freq_correction = audio_freq - rc_data.tone_freq;
 
   /* Calculate and write receiver
@@ -771,227 +952,15 @@ Tune_Tcvr( double x )
 	case K2: case K3:
 	  tcvr_freq += freq_correction; /* Now using CW-REV */
 	  break;
- 
+
   } /* switch( rc_data.tcvr_type ) */
-  if( !Write_Rx_Freq(tcvr_freq) ) return( FALSE );
+  Write_Rx_Freq( tcvr_freq );
 
   return( TRUE );
 } /* Tune_Tcvr() */
 
 /*-------------------------------------------------------------------------*/
 
-/*  Read_Rx_Freq()
- *
- *  Reads the Rx freq of the Yaesu transceiver.
- */
-
-  gboolean
-Read_Rx_Freq( int *freq )
-{
-  /* Abort if CAT disabled */
-  if( isFlagClear(CAT_SETUP) ) return( TRUE );
-
-  /* A char string for sending and receiving  */
-  /* data strings to and from the transceiver */
-  char cmnd_parm[5];
-
-  switch( rc_data.tcvr_type )
-  {
-	case FT847: case FT857:
-	/* Read Rx frequency */
-	bzero( cmnd_parm, sizeof(cmnd_parm) );
-	if( !Write_Tcvr_Serial(FT847_MAIN_VFO, sizeof(FT847_MAIN_VFO)) ||
-		!Read_Tcvr_Serial(cmnd_parm, sizeof(cmnd_parm)) )
-	  return( FALSE );
-
-	/* Decode frequency data to 10Hz */
-	*freq = 0;
-	*freq += (cmnd_parm[0] & 0xF0) >> 4;
-	*freq *= 10;
-	*freq += cmnd_parm[0] & 0x0F;
-	*freq *= 10;
-	*freq += (cmnd_parm[1] & 0xF0) >> 4;
-	*freq *= 10;
-	*freq += cmnd_parm[1] & 0x0F;
-	*freq *= 10;
-	*freq += (cmnd_parm[2] & 0xF0) >> 4;
-	*freq *= 10;
-	*freq += cmnd_parm[2] & 0x0F;
-	*freq *= 10;
-	*freq += (cmnd_parm[3] & 0xF0) >> 4;
-	*freq *= 10;
-	*freq += cmnd_parm[3] & 0x0F;
-	*freq *= 10;
-	break;
-
-	case K2: case K3:
-	/* Read Rx frequency */
-	if( !Write_Tcvr_Serial(K3_VFOA_G, sizeof(K3_VFOA_G)-1) ||
-		!Read_Tcvr_Serial(K3_VFOA_S, sizeof(K3_VFOA_S)-1) )
-		return( FALSE );
-	  *freq = atoi( K3_VFOA_S+2 );
-	  break;
-
-  } /* switch( rc_data.tcvr_type ) */
-
-  return( TRUE );
-} /* End of Read_Rx_Freq() */
-
-/*------------------------------------------------------------------------*/
-
-/*  Write_Rx_Freq()
- *
- *  Writes the Rx freq to the Yaesu transceiver.
- */
-
-  gboolean
-Write_Rx_Freq( int freq )
-{
-  /* Buffer used for converting freq. to string */
-  char freq_buff[9];
-  int idx; /* Index for loops etc */
-
-
-  /* Abort if CAT disabled */
-  if( isFlagClear(CAT_SETUP) ) return( TRUE );
-
-  switch( rc_data.tcvr_type )
-  {
-	case FT847: case FT857:
-	  /* Set Rx frequency */
-	  bzero( FT847_SET_VFO, 4 );
-	  snprintf( freq_buff, sizeof(freq_buff), "%08d", freq/10 );
-	  for( idx = 0; idx < 4; idx++ )
-	  {
-		FT847_SET_VFO[idx]  = (char)((freq_buff[2*idx]   - '0') << 4);
-		FT847_SET_VFO[idx] |= (freq_buff[2*idx+1] - '0');
-	  }
-	  if( !Write_Tcvr_Serial(FT847_SET_VFO, sizeof(FT847_SET_VFO)) )
-		return( FALSE );
-	  break;
-
-	case K2: case K3:
-	  /* Set Rx frequency */
-	  snprintf( K3_VFOA_S+2, sizeof(K3_VFOA_S)-2, "%011d;", freq );
-	  if( !Write_Tcvr_Serial(K3_VFOA_S, sizeof(K3_VFOA_S)-1) )
-		return( FALSE );
-	  break;
-
-  } /* switch( rc_data.tcvr_type ) */
-
-  return( TRUE );
-} /* Write_Rx_Freq() */
-
-/*-------------------------------------------------------------------------*/
-
-/* RS232_RTS_DTR_Set()
- *
- * Sets the Serial Port RTS/DTR line UP/DOWN
- */
-  gboolean
-RS232_RTS_DTR_Set( gboolean set )
-{
-  int modem_lines;
-
-  /* Abort if CAT disabled */
-  if( isFlagClear(RTS_DTR_SETUP) ) return( TRUE );
-
-  /* Read current modem state */
-  if( ioctl(serial_fd, TIOCMGET, &modem_lines) == -1 )
-  {
-	perror( rc_data.rts_serial );
-	rc_data.tcvr_type = NONE;
-	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
-		  lookup_widget(gbl_popup_menu, "none")), TRUE );
-	Error_Dialog(
-		_("Failed to set RTS/DTR\n"\
-		  "RTS/DTR Control disabled"), OK );
-	ClearFlag( RTS_DTR_SETUP );
-	return( FALSE );
-  }
-
-  /* Change the RTS/DTR state */
-  if( set )
-	modem_lines |=  (TIOCM_RTS | TIOCM_DTR);
-  else
-	modem_lines &= ~(TIOCM_RTS | TIOCM_DTR);
-
-  /* Write the new setting */
-  if( ioctl(serial_fd, TIOCMSET, &modem_lines) == -1 )
-  {
-	perror( rc_data.rts_serial );
-	rc_data.tcvr_type = NONE;
-	gtk_check_menu_item_set_active( GTK_CHECK_MENU_ITEM(
-		  lookup_widget(gbl_popup_menu, "none")), TRUE );
-	Error_Dialog(
-		_("Failed to set RTS/DTR\n"\
-		  "RTS/DTR Control disabled"), OK );
-	ClearFlag( RTS_DTR_SETUP );
-	return( FALSE );
-  }
-
-  return( TRUE );
-} /* RS232_RTS_DTR_Set() */
-
-/*-------------------------------------------------------------------------*/
-
-/* Tcvr_Status()
- * 
- * Gets or Sets Transceiver status (only for the K2/K3 currently)
- * Gets or Sets the Tx Power O/P, Mic Gain and Compressor level
- */
-  gboolean
-Tcvr_Status( int action, tcvr_status_t *status )
-{
-  /* For K2 or K3 */
-  if( (rc_data.tcvr_type == K2) || (rc_data.tcvr_type == K3) )
-  {
-	/* Get transceiver status */
-	if( action == GET_TCVR_STATUS )
-	{
-	  if( Write_Tcvr_Serial(K3_POWER_G, sizeof(K3_POWER_G)-1) &&
-		  Read_Tcvr_Serial(K3_POWER_S, sizeof(K3_POWER_S)-1) &&
-		  (rc_data.tcvr_type == K3 ?
-		   Write_Tcvr_Serial(K3_COMP_G, sizeof(K3_COMP_G)-1) : TRUE) &&
-		  (rc_data.tcvr_type == K3 ?
-		   Read_Tcvr_Serial(K3_COMP_S, sizeof(K3_COMP_S)-1) : TRUE) &&
-		  Write_Tcvr_Serial(K3_VFOA_G, sizeof(K3_VFOA_G)-1) &&
-		  Read_Tcvr_Serial(K3_VFOA_S, sizeof(K3_VFOA_S)-1) &&
-		  Write_Tcvr_Serial(K3_MODE_G, sizeof(K3_MODE_G)-1) &&
-		  Read_Tcvr_Serial(K3_MODE_S, sizeof(K3_MODE_S)-1) )
-	  {
-		status->tx_power = atoi( K3_POWER_S+2 );
-		status->comp_lev = atoi(  K3_COMP_S+2 );
-		status->rig_mode = atoi(  K3_MODE_S+2 );
-		status->vfo_freq = atoi(  K3_VFOA_S+2 );
-		return( TRUE );
-	  }
-	} /* if( action == GET_TCVR_STATUS ) */
-
-	/* Set transceiver status */
-	if( action == SET_TCVR_STATUS )
-	{
-	  snprintf( K3_POWER_S+2, sizeof(K3_POWER_S)-2,  "%03d;", status->tx_power );
-	  snprintf(  K3_COMP_S+2, sizeof(K3_COMP_S)-2,  "%03d;", status->comp_lev );
-	  snprintf(  K3_MODE_S+2, sizeof(K3_MODE_S)-2,    "%d;", status->rig_mode );
-	  snprintf(  K3_VFOA_S+2, sizeof(K3_VFOA_S)-2, "%011d;", status->vfo_freq );
-
-	  if( ((unsigned int)Write_Tcvr_Serial(K3_VFOA_S, sizeof(K3_VFOA_S)-1) | sleep(1) ) &&
-		  Write_Tcvr_Serial(K3_MODE_S, sizeof(K3_MODE_S)-1) &&
-		  Write_Tcvr_Serial(K3_VFOA_S, sizeof(K3_VFOA_S)-1) &&
-		  (rc_data.tcvr_type == K3 ?
-		   Write_Tcvr_Serial(K3_COMP_S, sizeof(K3_COMP_S)-1) : TRUE ) &&
-		  Write_Tcvr_Serial(K3_POWER_S, sizeof(K3_POWER_S)-1) )
-		return( TRUE );
-
-	} /* if( action == SET_TCVR_STATUS ) */
-  } /* if( (rc_data.tcvr_type == K2) || (rc_data.tcvr_type == K2) ) */
-
-  return( FALSE );  
-} /* Tcvr_Status() */
-
-/*-------------------------------------------------------------------------*/
-
 /* Set_Tx_Power()
  *
  * Sets the Tx power output and speech
@@ -1005,31 +974,27 @@ Set_Tx_Power( int tx_power, int comp_lev )
   if( (rc_data.tcvr_type == K2) || (rc_data.tcvr_type == K3) )
   {
 	/* Read transceiver Tx power setting */
-	if( !Write_Tcvr_Serial(K3_POWER_G, sizeof(K3_POWER_G)-1) ||
-		!Read_Tcvr_Serial( K3_POWER_S, sizeof(K3_POWER_S)-1) )
-	  return( FALSE );
+	Write_Tcvr_Serial( K3_POWER_G, sizeof(K3_POWER_G)-1 );
+	Read_Tcvr_Serial(  K3_POWER_S, sizeof(K3_POWER_S)-1 );
 	power = atoi( K3_POWER_S+2 );
 
 	/* Change power setting if request is different */
 	if( power != tx_power )
 	{
 	  snprintf( K3_POWER_S+2, sizeof(K3_POWER_S)-2, "%03d;", tx_power );
-	  if( !Write_Tcvr_Serial(K3_POWER_S, sizeof(K3_POWER_S)-1) )
-		return( FALSE );
+	  Write_Tcvr_Serial( K3_POWER_S, sizeof(K3_POWER_S)-1 );
 	}
 
 	/* Read transceiver compressor setting */
-	if( !Write_Tcvr_Serial(K3_COMP_G, sizeof(K3_COMP_G)-1) ||
-		!Read_Tcvr_Serial( K3_COMP_S, sizeof(K3_COMP_S)-1) )
-	  return( FALSE );
+	Write_Tcvr_Serial( K3_COMP_G, sizeof(K3_COMP_G)-1 );
+	Read_Tcvr_Serial(  K3_COMP_S, sizeof(K3_COMP_S)-1 );
 	comp = atoi( K3_COMP_S+2 );
 
 	/* Change compressor setting if new request is different */
 	if( (comp != comp_lev) && (rc_data.tcvr_type == K3) )
 	{
 	  snprintf( K3_COMP_S+2, sizeof(K3_COMP_S)-2, "%03d;", comp_lev );
-	  if( !Write_Tcvr_Serial(K3_COMP_S, sizeof(K3_COMP_S)-1) )
-		return( FALSE );
+	  Write_Tcvr_Serial( K3_COMP_S, sizeof(K3_COMP_S)-1 );
 	}
 
   } /* if( (rc_data.tcvr_type == K2) || (rc_data.tcvr_type == K3) ) */
