@@ -1,8 +1,4 @@
 /*
- *  xfhell: An application to transmit and receive
- *  Feld Hell signals using a computer's sound card
- *
- *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
  *  published by the Free Software Foundation; either version 3 of
@@ -22,7 +18,6 @@
 #include <sys/ioctl.h>
 #include <termios.h>
 #include "common.h"
-#include "support.h"
 
 /* Transceiver type */
 enum
@@ -48,5 +43,9 @@ enum
 /* Maximum number of serial port retries before error condition */
 #define MAX_SERIAL_RETRIES	10
 
+/* Range (+/-) in pixels to look for IFFT maximum
+ * when click-to-tune on waterfall by user */
+#define CLICK_TUNE_RANGE	10
+
 #endif
 
