@@ -1,13 +1,4 @@
-/*  codec.c
- *
- *  Feld Hell coder/decoder functions of xfhell application
- */
-
 /*
- *  xfhell: An application to transmit and receive
- *  Feld Hell signals using a computer's sound card
- *
- *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
  *  published by the Free Software Foundation; either version 3 of
@@ -67,7 +58,7 @@ Assemble_Column( unsigned char *column )
 
 
   /* Allocate memory to row_average */
-  if( fbb_h != rc_data.font_data.fbb_h )
+  if( !fbb_h || (fbb_h != rc_data.font_data.fbb_h) )
   {
 	fbb_h  = rc_data.font_data.fbb_h;
 	fbb_h2 = fbb_h / 2;
@@ -208,34 +199,12 @@ Assemble_Column( unsigned char *column )
 
 /*------------------------------------------------------------------------*/
 
-/* Transmit_Character()
- *
- * Transmits one Hell character pointed by "glx"
- */
-
-  gboolean
-Transmit_Character( font_data_t *font_data, int glx )
-{
-  gboolean ret;
-
-  if( isFlagSet(MODE_FELDHELL) )
-	ret = Tx_Feld_Character(font_data, glx);
-  else
-	ret = Tx_FMHell_Character(font_data, glx);
-
-  if( !ret ) Close_Playback();
-
-  return( ret );
-} /* Transmit_Character() */
-
-/*------------------------------------------------------------------------*/
-
 /* Tx_Feld_Character()
  *
  * Transmits one Feld Hell character pointed by "glx"
  */
 
-  gboolean
+  static gboolean
 Tx_Feld_Character( font_data_t *font_data, int glx )
 {
   int
@@ -292,7 +261,6 @@ Tx_Feld_Character( font_data_t *font_data, int glx )
 	  last_dot = 0;
 
 	/* Go up in bitmap buffer row by row */
-	bmy = font_data->fbb_h;
 	for( bmy = font_data->fbb_h - 1; bmy >= 0; bmy-- )
 	{
 	  /* Set tone control according to bitmap value. */
@@ -332,7 +300,7 @@ Tx_Feld_Character( font_data_t *font_data, int glx )
 		  /* Fill buffer with cosine shaped edge for half dot */
 		  for( cosine_idx = 0; cosine_idx < half_dot; cosine_idx++ )
 		  {
-			xmit_buffer[buff_idx] = 
+			xmit_buffer[buff_idx] =
 			  (cosine_table[cosine_idx] * sine_table[sine_idx]) >> 15;
 			buff_idx += rc_data.num_chn;
 			if( ++sine_idx >= rc_data.samples_per_cycle )
@@ -413,7 +381,7 @@ Tx_Feld_Character( font_data_t *font_data, int glx )
  * Transmits one FM Hell character pointed by "glx"
  */
 
-  gboolean
+  static gboolean
 Tx_FMHell_Character( font_data_t *font_data, int glx )
 {
   int
@@ -453,7 +421,8 @@ Tx_FMHell_Character( font_data_t *font_data, int glx )
   }
 
   /* Initialize on new parameters or first call */
-  if( baud_rate != rc_data.baud_rate )
+  if( (baud_rate < rc_data.baud_rate) ||
+	  (baud_rate > rc_data.baud_rate) )
   {
 	baud_rate = rc_data.baud_rate;
 
@@ -531,6 +500,28 @@ Tx_FMHell_Character( font_data_t *font_data, int glx )
 
 /*------------------------------------------------------------------------*/
 
+/* Transmit_Character()
+ *
+ * Transmits one Hell character pointed by "glx"
+ */
+
+  gboolean
+Transmit_Character( font_data_t *font_data, int glx )
+{
+  gboolean ret;
+
+  if( isFlagSet(MODE_FELDHELL) )
+	ret = Tx_Feld_Character(font_data, glx);
+  else
+	ret = Tx_FMHell_Character(font_data, glx);
+
+  if( !ret ) Close_Playback();
+
+  return( ret );
+} /* Transmit_Character() */
+
+/*------------------------------------------------------------------------*/
+
 /* Morse_Transmit()
  *
  * Transmits a string Morse code
@@ -570,7 +561,7 @@ Morse_Transmit( char *mesg )
 
   /* Allocate memory to samples buffer */
   int tmp = 3 * rc_data.num_chn * rc_data.dsp_rate / rc_data.wpm;
-  if( buf_size != tmp )
+  if( !buf_size || (buf_size != tmp) )
   {
 	buf_size = tmp;
 	if( !mem_realloc((void **)&samples_buff,
