@@ -1,13 +1,4 @@
-/*  common.h
- *
- *  Header file for xfhell
- */
-
 /*
- *  xfhell: An application to transmit and receive
- *  Feld Hell signals using a computer's sound card
- *
- *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
  *  published by the Free Software Foundation; either version 3 of
@@ -25,9 +16,17 @@
 #ifndef COMMON_H
 #define COMMON_H	 1
 
+#ifdef HAVE_CONFIG_H
+#  include <config.h>
+#endif
+
 #include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
 #include <alsa/asoundlib.h>
+#include <unistd.h>
+#include <string.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <math.h>
 
 /* General definitions for signal processing */
 #define KEY_BUFF_SIZE		 256 /* Keyboard buffer size */
@@ -42,9 +41,6 @@
 /* Signal Detector parameter definitions */
 #define ELEMENT_MAX			100	/* Defined max val of dot elements */
 
-/* DFT parameter definitions */
-#define DFT_INPUT_SIZE	1020 /* Size of dft input samples array */
-
 /* Flag definitions */
 #define NO_CALLBACKS		0x00000001U	/* Don't respond to callback if needed */
 #define RECORD_QSO			0x00000002	/* Enable recording of QSO's */
@@ -164,12 +160,14 @@ typedef struct
 	dot_size,			/* Receive dot height in pixels */
 	bitmap_height,		/* Receive bitmap height in pixels */
 	bitmap_width,		/* Receive bitmap width in pixels  */
+	rxarea_width,		/* Rx drawingarea width  */
+	rxarea_height,		/* Rx drawingarea height */
 	num_rows,			/* Number of character rows in Rx window */
 	samples_per_cycle,	/* Samples per cycle of audio freq. */
 	samples_per_dot,	/* Samples per Hell character half dot */
 	tone_freq,			/* Input tone (Rx BFO) frequency */
-	dft_stride,			/* DFT's stride over input data  */
-	center_line,		/* Position of tuning reference in dft window */
+	ifft_stride,			/* IFFT's stride over input data  */
+	center_line,		/* Position of tuning reference in ifft window */
 	txbuff_len,			/* Tx signal samples buffer size */
 	rxbuff_len;			/* Rx signal samples buffer size */
 
@@ -203,6 +201,9 @@ typedef struct
   /* Font data buffer */
   font_data_t font_data;
 
+  /* Glade file path */
+  char xfhell_glade[64];
+
 } rc_data_t;
 
 /* QSO record type definition */
@@ -234,7 +235,6 @@ typedef struct
 typedef struct
 {
   GdkPixbuf *pixbuf; /* The GDK pixbuf */
-
   guchar *pixels;	 /* Pointer to the pixels in above */
 
   int
@@ -260,29 +260,54 @@ typedef struct
 
 /*-------------------------------------------------------------------*/
 
+/*
+ * Standard gettext macros.
+ */
+#ifdef ENABLE_NLS
+#  include <libintl.h>
+#  undef _
+#  define _(String) dgettext (PACKAGE, String)
+#  define Q_(String) g_strip_context ((String), gettext (String))
+#  ifdef gettext_noop
+#    define N_(String) gettext_noop (String)
+#  else
+#    define N_(String) (String)
+#  endif
+#else
+#  define textdomain(String) (String)
+#  define gettext(String) (String)
+#  define dgettext(Domain,Message) (Message)
+#  define dcgettext(Domain,Message,Type) (Message)
+#  define bindtextdomain(Domain,Directory) (Domain)
+#  define _(String) (String)
+#  define Q_(String) g_strip_context ((String), (String))
+#  define N_(String) (String)
+#endif
+
+/*-------------------------------------------------------------------*/
+
 /* Function prototypes produced by cproto */
 /* callbacks.c */
-void Error_Dialog(char *message, gboolean hide);
-void Quit_Dialog(char *message);
-void Save_Dialog(char *message);
-void Bad_Entry_Dialog(char *message);
-void on_main_window_destroy(GtkObject *object, gpointer user_data);
+void Error_Dialog(char *mesg, gboolean hide);
+void Quit_Dialog(char *mesg);
+void Save_Dialog(char *mesg);
+void Bad_Entry_Dialog(char *mesg);
+void on_main_window_destroy(GObject *object, gpointer user_data);
 gboolean on_main_window_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
 gboolean on_tx_textview_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
 gboolean on_tx_textview_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
-void on_xmit_clicked(GtkButton *button, gpointer user_data);
 void on_txvol_spinbutton_value_changed(GtkSpinButton *spinbutton, gpointer user_data);
-gboolean on_drawingarea_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer user_data);
-gboolean on_drawingarea_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
-void on_drawingarea_size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data);
+gboolean on_rx_drawingarea_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer user_data);
+gboolean on_rx_drawingarea_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
+void on_xmit_clicked(GtkButton *button, gpointer user_data);
 void on_rcve_clicked(GtkButton *button, gpointer user_data);
 void on_deskew_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 void on_height_checkbutton_toggled(GtkToggleButton *togglebutton, gpointer user_data);
 void on_rxvol_spinbutton_value_changed(GtkSpinButton *spinbutton, gpointer user_data);
 void on_contrast_value_changed(GtkRange *range, gpointer user_data);
-gboolean on_scope_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer user_data);
+gboolean on_scope_draw(GtkWidget *widget, cairo_t *cr, gpointer user_data);
 gboolean on_scope_button_press_event(GtkWidget *widget, GdkEventButton *event, gpointer user_data);
-void on_scope_size_allocate(GtkWidget *widget, GdkRectangle *allocation, gpointer user_data);
+void on_scope_configure_event(GtkWidget *widget, GdkEventConfigure *event, gpointer user_data);
 void on_rst_out_changed(GtkEditable *editable, gpointer user_data);
 void on_rst_in_changed(GtkEditable *editable, gpointer user_data);
 void on_op_name_changed(GtkEditable *editable, gpointer user_data);
@@ -304,7 +329,7 @@ void on_macro7_clicked(GtkButton *button, gpointer user_data);
 void on_macro8_clicked(GtkButton *button, gpointer user_data);
 void on_macro9_clicked(GtkButton *button, gpointer user_data);
 void on_macro10_clicked(GtkButton *button, gpointer user_data);
-void on_error_dialog_destroy(GtkObject *object, gpointer user_data);
+void on_error_dialog_destroy(GObject *object, gpointer user_data);
 gboolean on_error_dialog_delete_event(GtkWidget *widget, GdkEvent *event, gpointer user_data);
 void on_error_quit_button_clicked(GtkButton *button, gpointer user_data);
 void on_error_ok_button_clicked(GtkButton *button, gpointer user_data);
@@ -314,6 +339,7 @@ void on_feldhell_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_fmhell_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_bdrate_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_font_selector_activate(GtkMenuItem *menuitem, gpointer user_data);
+void on_filechooser_destroy(GObject *object, gpointer user_data);
 void on_show_font_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_reverse_video_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_k2_activate(GtkMenuItem *menuitem, gpointer user_data);
@@ -322,37 +348,32 @@ void on_ft847_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_ft857_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_rts_ctl_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_none_activate(GtkMenuItem *menuitem, gpointer user_data);
-void on_md2x4_activate(GtkMenuItem *menuitem, gpointer user_data);
-void on_md1x2_activate(GtkMenuItem *menuitem, gpointer user_data);
+void on_md2x2_activate(GtkMenuItem *menuitem, gpointer user_data);
+void on_md3x3_activate(GtkMenuItem *menuitem, gpointer user_data);
+void on_md4x4_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_record_qsos_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_capitalize_letters_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_loopback_tx_to_rx_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_identify_in_cw_activate(GtkMenuItem *menuitem, gpointer user_data);
 void on_quit_activate(GtkMenuItem *menuitem, gpointer user_data);
-void on_bad_entry_ok_clicked(GtkButton *button, gpointer user_data);
+void on_entry_ok_clicked(GtkButton *button, gpointer user_data);
 void on_cancel_quit_clicked(GtkButton *button, gpointer user_data);
 void on_quit_ok_clicked(GtkButton *button, gpointer user_data);
-void on_fileselection_response(GtkDialog *dialog, gint response_id, gpointer user_data);
+void on_filechooser_response(GtkDialog *dialog, gint response_id, gpointer user_data);
+void on_scope_button_clicked(GtkButton *button, gpointer user_data);
+void on_rx_drawingarea_configure_event(GtkWidget *widget, GdkEventConfigure *event, gpointer user_data);
+gboolean on_rx_drawingarea_draw(GtkWidget *widget, cairo_t *cr, gpointer user_data);
 /* cat.c */
 gboolean Open_Tcvr_Serial(void);
 void Close_Tcvr_Serial(void);
 gboolean Open_RTS_DTR_Serial(void);
-void Cancel_CAT(void);
-gboolean Read_Tcvr_Serial(char *data, size_t len);
-gboolean Write_Tcvr_Serial(char *data, size_t len);
 gboolean Read_Rx_Status(char *freq, char *rst);
 gboolean Set_Tcvr_Mode(void);
 gboolean Tune_Tcvr(double x);
-gboolean Read_Rx_Freq(int *freq);
-gboolean Write_Rx_Freq(int freq);
-gboolean RS232_RTS_DTR_Set(gboolean set);
-gboolean Tcvr_Status(int action, tcvr_status_t *status);
 gboolean Set_Tx_Power(int tx_power, int comp_lev);
 /* codec.c */
 gboolean Assemble_Column(unsigned char *column);
 gboolean Transmit_Character(font_data_t *font_data, int glx);
-gboolean Tx_FMHell_Character(font_data_t *font_data, int glx);
-gboolean Tx_Feld_Character(font_data_t *font_data, int glx);
 gboolean Morse_Transmit(char *mesg);
 gboolean Make_Sin_Wavetable(void);
 gboolean Make_Cos_Wavetable(void);
@@ -363,36 +384,34 @@ gboolean FMHell_Level(int *elem_lev);
 /* display.c */
 gboolean Draw_Column(unsigned char *column);
 void Display_Waterfall(void);
-gboolean Create_Pixbuf(GtkWidget *drawingarea);
-void Clear_Pixbuf(pixbuf_data_t *pixbuf, guint32 fg);
+gboolean Create_Rx_Pixbuf(void);
+gboolean Create_Scope_Pixbuf(int width, int height);
 void Print_Character(int printchr, GtkTextBuffer *text_buffer, GtkWidget *scrolledwindow, GtkTextIter *iter);
-void Display_Signal(int plot);
+void Display_Signal(cairo_t *cr);
 void Set_TxRx_Labels(void);
-/* dft.c */
-void Idft_Init(int dft_input_size, int dft_bin_size);
-void Idft(int dft_input_size, int dft_bin_size);
+/* ifft.c */
+gboolean Initialize_IFFT(int16_t width);
+void IFFT_Real(int16_t *data );
+void IFFT_Data(int16_t sample);
 /* interface.c */
-GtkWidget *create_main_window(void);
-GtkWidget *create_error_dialog(void);
-GtkWidget *create_save_dialog(void);
-GtkWidget *create_main_menu(void);
-GtkWidget *create_bad_entry_dialog(void);
-GtkWidget *create_quit_dialog(void);
-GtkWidget *create_fileselection(void);
+GtkWidget *Builder_Get_Object(GtkBuilder *builder, gchar *name);
+GtkWidget *create_main_window(GtkBuilder **builder);
+GtkWidget *create_error_dialog(GtkBuilder **builder);
+GtkWidget *create_save_dialog(GtkBuilder **builder);
+GtkWidget *create_popup_menu(GtkBuilder **builder);
+GtkWidget *create_entry_dialog(GtkBuilder **builder);
+GtkWidget *create_quit_dialog(GtkBuilder **builder);
+GtkWidget *create_filechooser(GtkBuilder **builder);
 /* main.c */
 int main(int argc, char *argv[]);
-gboolean Load_Config(gpointer data);
 /* operation.c */
 gboolean Transmit_Keybd(gpointer data);
 gboolean Transmit_Macro(gpointer data);
 gboolean Receive_Mode(gpointer data);
-gboolean Initialize_Transmit(gpointer data);
 /* shared.c */
 /* sound.c */
-gboolean Open_PCM(snd_pcm_t **handle, snd_pcm_stream_t stream, char *mesg, int *error);
 gboolean Open_Capture(char *mesg, int *error);
 gboolean Open_Playback(char *mesg, int *error);
-gboolean Open_Mixer(char *mesg, int *error);
 gboolean Set_Playback_Level(int level, char *mesg, int *error);
 gboolean Set_Capture_Level(int level, char *mesg, int *error);
 void Close_Playback(void);
@@ -400,32 +419,17 @@ void Close_Capture(void);
 void Close_Mixer(void);
 gboolean Signal_Sample(short *sample_val);
 gboolean DSP_Write(short *buffer, int buff_size);
-gboolean Xrun_Recovery(snd_pcm_t *handle, int error);
-/* support.c */
-GtkWidget *lookup_widget(GtkWidget *widget, const gchar *widget_name);
-void add_pixmap_directory(const gchar *directory);
-GtkWidget *create_pixmap(GtkWidget *widget, const gchar *filename);
-GdkPixbuf *create_pixbuf(const gchar *filename);
-void glade_set_atk_action_description(AtkAction *action, const gchar *action_name, const gchar *description);
 /* utils.c */
+gboolean Load_Config(gpointer data);
 gboolean Save_QSO_Record(void);
 gboolean Read_QSO_Record(void);
 void Clear_Record_Fields(gboolean all);
 gboolean Open_Record_File(void);
-gboolean Load_Fonts(char *font_file);
-gboolean Find_Line(char *line, const char *str, FILE *fp);
-void Find_String(char *line, int *idx);
 unsigned int Hex2Bitmap(char *hex_str);
 gboolean Render_Font(font_data_t *font, int glx);
 gboolean Save_Pixbuf(void);
-gboolean Alloc_Tx_Buffer(void);
 void Process_RST(GtkEditable *editable);
 void Select_Macro(int macro_num);
-int Open_File(FILE **fp, char *fpath, const char *mode);
-void Close_File(FILE **fp);
-gboolean File_Print(FILE **fp, char *string);
-gboolean Locate_Line(char **line_idx, char *line, const char *string, FILE *fp);
-gboolean Locate_String(char **line_idx, char *line, const char *string);
 void New_Parameters(void);
 gboolean mem_alloc(void **ptr, size_t req);
 gboolean mem_realloc(void **ptr, size_t req);
@@ -436,11 +440,10 @@ void SetFlag(long flag);
 void ClearFlag(long flag);
 void ToggleFlag(long flag);
 void Cleanup(void);
-int Load_Line(char *buff, FILE *pfile, char *messg);
 void Usage(void);
 void Strlcpy(char *dest, const char *src, size_t n);
 void Strlcat(char *dest, const char *src, size_t n);
-double Atof(const char *nptr);
+double Ato(const char *nptr);
 
-#endif /* End of xfhell.h */
+#endif
 
