@@ -1,8 +1,4 @@
 /*
- *  xfhell: An application to transmit and receive
- *  Feld Hell signals using a computer's sound card
- *
- *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
  *  published by the Free Software Foundation; either version 3 of
@@ -20,11 +16,10 @@
 #define DISPLAY_H	1
 
 #include "common.h"
-#include "support.h"
 
 /* Length and multiplier of amplitude averaging window  */
-#define AMPL_AVE_WIN		2
-#define AMPL_AVE_MUL		1
+#define AMPL_AVE_WIN		3
+#define AMPL_AVE_MUL		2
 
 /* Receive and Transmit status indicators */
 #define RECV_OFF	_("<span background=\"lightgrey\"> RECEIVE (OFF) </span>")
@@ -33,5 +28,8 @@
 #define XMIT_OFF	_("<span background=\"lightgrey\"> TRANSMIT (OFF) </span>")
 #define XMIT_ON		_("<span background=\"red\" foreground =\"white\"> TRANSMIT (ON) </span>")
 
+/* Scale FFT output bins */
+#define IFFT_SCALE		64
+
 #endif
 
