@@ -1,8 +1,4 @@
 /*
- *  xfhell: An application to transmit and receive
- *  Feld Hell signals using a computer's sound card
- *
- *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
  *  published by the Free Software Foundation; either version 3 of
@@ -24,9 +20,7 @@
 #endif
 
 #include "common.h"
-#include "cat.h"
 #include "interface.h"
-#include "support.h"
 
 #endif
 
