@@ -1,13 +1,4 @@
-/*  sound.c
- *
- *  Soundcard handling functions of xfhell application
- */
-
 /*
- *  xfhell: An application to transmit and receive
- *  Feld Hell signals using a computer's sound card
- *
- *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
  *  published by the Free Software Foundation; either version 3 of
@@ -47,7 +38,7 @@ static short *recv_buffer = NULL;
  *
  * Opens a pcm device for a given handle
  */
-  gboolean
+  static gboolean
 Open_PCM(
 	snd_pcm_t **handle,
 	snd_pcm_stream_t stream,
@@ -160,81 +151,11 @@ Open_PCM(
 
 /*------------------------------------------------------------------------*/
 
-/* Open_Capture()
- *
- * Opens sound card for Capture
- */
-  gboolean
-Open_Capture( char *mesg, int *error )
-{
-  /* Return if Capture is setup */
-  if( isFlagSet(CAPTURE_SETUP) ) return( TRUE );
-
-  /* Open & setup pcm for Capture */
-  if( !Open_PCM(
-		&capture_handle,
-		SND_PCM_STREAM_CAPTURE,
-		mesg, error) )
-	return( FALSE );
-
-  /* Size of samples buffer depends on stereo/mono mode */
-  rc_data.rxbuff_len = PERIOD_SIZE * rc_data.num_chn;
-  
-  /* Index to rx samples buffer (set to end) */
-  rx_buff_idx = rc_data.rxbuff_len;
-
-  /* Allocate memory to receive samples buffer */
-  if( recv_buffer == NULL )
-	if( !mem_alloc((void **)&recv_buffer,
-		(size_t)rc_data.rxbuff_len * sizeof(short)) )
-	return( FALSE );
-  memset( recv_buffer, 0,
-	  (size_t)rc_data.rxbuff_len * sizeof(short) );
-
-  /* Open mixer & set playback voulume, abort on failure.
-   * Failure to set Volume levels is not taken as fatal */
-  if( !Open_Mixer(mesg, error) ) return( FALSE );
-  Set_Capture_Level( rc_data.cap_lev, mesg, error );
-
-  SetFlag( CAPTURE_SETUP );
-  return( TRUE );
-} /* Open_Capture() */
-
-/*------------------------------------------------------------------------*/
-
-/* Open_Playback()
- *
- * Opens sound card for Playback
- */
-  gboolean
-Open_Playback( char *mesg, int *error )
-{
-  /* Return if Playback is setup */
-  if( isFlagSet(PLAYBACK_SETUP) ) return( TRUE );
-
-  /* Open & setup pcm for Playback */
-  if( !Open_PCM(
-		&playback_handle,
-		SND_PCM_STREAM_PLAYBACK,
-		mesg, error) )
-	return( FALSE );
-
-  /* Open mixer & set playback volume, abort on failure
-   * Failure to set Volume levels is not taken as fatal */
-  if( !Open_Mixer(mesg, error) ) return( FALSE );
-  Set_Playback_Level( rc_data.pbk_lev, mesg, error );
-
-  SetFlag( PLAYBACK_SETUP );
-  return( TRUE );
-} /* Open_Playback() */
-
-/*------------------------------------------------------------------------*/
-
 /* Open_Mixer()
  *
  * Opens mixer interface
  */
-  gboolean
+  static gboolean
 Open_Mixer( char *mesg, int *error )
 {
   snd_mixer_elem_t *elem;
@@ -398,6 +319,76 @@ Open_Mixer( char *mesg, int *error )
 
 /*------------------------------------------------------------------------*/
 
+/* Open_Capture()
+ *
+ * Opens sound card for Capture
+ */
+  gboolean
+Open_Capture( char *mesg, int *error )
+{
+  /* Return if Capture is setup */
+  if( isFlagSet(CAPTURE_SETUP) ) return( TRUE );
+
+  /* Open & setup pcm for Capture */
+  if( !Open_PCM(
+		&capture_handle,
+		SND_PCM_STREAM_CAPTURE,
+		mesg, error) )
+	return( FALSE );
+
+  /* Size of samples buffer depends on stereo/mono mode */
+  rc_data.rxbuff_len = PERIOD_SIZE * rc_data.num_chn;
+
+  /* Index to rx samples buffer (set to end) */
+  rx_buff_idx = rc_data.rxbuff_len;
+
+  /* Allocate memory to receive samples buffer */
+  if( recv_buffer == NULL )
+	if( !mem_alloc((void **)&recv_buffer,
+		(size_t)rc_data.rxbuff_len * sizeof(short)) )
+	return( FALSE );
+  memset( recv_buffer, 0,
+	  (size_t)rc_data.rxbuff_len * sizeof(short) );
+
+  /* Open mixer & set playback voulume, abort on failure.
+   * Failure to set Volume levels is not taken as fatal */
+  if( !Open_Mixer(mesg, error) ) return( FALSE );
+  Set_Capture_Level( rc_data.cap_lev, mesg, error );
+
+  SetFlag( CAPTURE_SETUP );
+  return( TRUE );
+} /* Open_Capture() */
+
+/*------------------------------------------------------------------------*/
+
+/* Open_Playback()
+ *
+ * Opens sound card for Playback
+ */
+  gboolean
+Open_Playback( char *mesg, int *error )
+{
+  /* Return if Playback is setup */
+  if( isFlagSet(PLAYBACK_SETUP) ) return( TRUE );
+
+  /* Open & setup pcm for Playback */
+  if( !Open_PCM(
+		&playback_handle,
+		SND_PCM_STREAM_PLAYBACK,
+		mesg, error) )
+	return( FALSE );
+
+  /* Open mixer & set playback volume, abort on failure
+   * Failure to set Volume levels is not taken as fatal */
+  if( !Open_Mixer(mesg, error) ) return( FALSE );
+  Set_Playback_Level( rc_data.pbk_lev, mesg, error );
+
+  SetFlag( PLAYBACK_SETUP );
+  return( TRUE );
+} /* Open_Playback() */
+
+/*------------------------------------------------------------------------*/
+
 /* Set_Playback_Level()
  *
  * Sets Playback Control level
@@ -552,6 +543,50 @@ Close_Mixer( void )
 
 /*------------------------------------------------------------------------*/
 
+/* Xrun_Recovery()
+ *
+ * Recover from underrrun (broken pipe) and suspend
+ */
+  static gboolean
+Xrun_Recovery( snd_pcm_t *handle, int error )
+{
+  if( error == -EPIPE )
+  {
+	error = snd_pcm_prepare( handle );
+	if( error < 0 )
+	{
+	  char mesg[MESG_SIZE];
+	  snprintf( mesg, sizeof(mesg),
+		  _("Cannot recover from underrun, prepare failed\n"
+			"Error: %s\n"), snd_strerror(error) );
+	  Error_Dialog( mesg, QUIT );
+	  return( FALSE );
+	}
+  }
+  else if( error == -ESTRPIPE )
+  {
+	while( (error = snd_pcm_resume(handle)) == -EAGAIN )
+	  sleep(1);
+	if( error < 0 )
+	{
+	  error = snd_pcm_prepare( handle );
+	  if( error < 0 )
+	  {
+		char mesg[MESG_SIZE];
+		snprintf( mesg, sizeof(mesg),
+			_("Cannot recover from suspend, prepare failed\n"\
+			  "Error: %s\n"), snd_strerror(error) );
+		Error_Dialog( mesg, QUIT );
+		return( FALSE );
+	  }
+	}
+  }
+
+  return( TRUE );
+} /* Xrun_Recovery() */
+
+/*------------------------------------------------------------------------*/
+
 /*  Signal_Sample()
  *
  *  Gets the next DSP sample of the signal input.
@@ -562,10 +597,6 @@ Signal_Sample( short *sample_val )
 {
   snd_pcm_sframes_t error;
 
-  static int
-	dft_idx = 0, /* dft input buffer idx */
-	cnt = 0;	 /* Count of calls to this function */
-
   /* Read samples from DSP if in Receive mode */
   if( isFlagSet(RECEIVE_MODE) && isFlagClear(KEYBD_BUSY) )
   {
@@ -603,32 +634,9 @@ Signal_Sample( short *sample_val )
   } /* if( isFlagSet(RECEIVE_MODE) ) */
 
   /* Display waterfall when input buffer full.
-   * dft_stride samples are added for each input element */
+   * ifft_stride samples are added for each input element */
   if( isFlagClear(ENABLE_SCOPE) )
-  {
-	/* Summate (decimate) samples for the DFT */
-	dft_in_r[dft_idx] += *sample_val;
-
-	/* Reset stride (decimation) counter */
-	if( ++cnt >= rc_data.dft_stride )
-	{
-	  cnt = 0;
-
-	  /* Normalize DFT input samples */
-	  dft_in_r[dft_idx] /= rc_data.dft_stride;
-	  dft_idx++;
-
-	  /* Display waterfall when input buffer full */
-	  if( dft_idx >= DFT_INPUT_SIZE )
-	  {
-		Display_Waterfall();
-		dft_idx = 0;
-	  }
-
-	  /* Clear for next summation */
-	  dft_in_r[dft_idx] = 0;
-	}
-  } /* if( isFlagClear(ENABLE_SCOPE) ) */
+	IFFT_Data( *sample_val );
 
   return( TRUE );
 } /* End of Signal_Sample() */
@@ -691,47 +699,3 @@ DSP_Write( short *buffer, int buff_size )
 
 /*------------------------------------------------------------------------*/
 
-/* Xrun_Recovery()
- *
- * Recover from underrrun (broken pipe) and suspend
- */
-  gboolean
-Xrun_Recovery( snd_pcm_t *handle, int error )
-{
-  if( error == -EPIPE )
-  {
-	error = snd_pcm_prepare( handle );
-	if( error < 0 )
-	{
-	  char mesg[MESG_SIZE];
-	  snprintf( mesg, sizeof(mesg),
-		  _("Cannot recover from underrun, prepare failed\n"
-			"Error: %s\n"), snd_strerror(error) );
-	  Error_Dialog( mesg, QUIT );
-	  return( FALSE );
-	}
-  }
-  else if( error == -ESTRPIPE )
-  {
-	while( (error = snd_pcm_resume(handle)) == -EAGAIN )
-	  sleep(1);
-	if( error < 0 )
-	{
-	  error = snd_pcm_prepare( handle );
-	  if( error < 0 )
-	  {
-		char mesg[MESG_SIZE];
-		snprintf( mesg, sizeof(mesg),
-			_("Cannot recover from suspend, prepare failed\n"\
-			  "Error: %s\n"), snd_strerror(error) );
-		Error_Dialog( mesg, QUIT );
-		return( FALSE );
-	  }
-	}
-  }
-
-  return( TRUE );
-} /* Xrun_Recovery() */
-
-/*------------------------------------------------------------------------*/
-
