@@ -1,8 +1,4 @@
 /*
- *  xfhell: An application to transmit and receive
- *  Feld Hell signals using a computer's sound card
- *
- *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License as
  *  published by the Free Software Foundation; either version 3 of
@@ -20,7 +16,6 @@
 #define SOUND_H		1
 
 #include "common.h"
-#include "support.h"
 
 #define NUM_PERIODS			   4 /* Number of periods */
 #define PERIOD_SIZE			4096 /* PCM period size */
